/* BIG GUN: Naval Gun Simulator, beta v. 1.1 (5/16/01; 4/25/2001) */


	/*****

		This program is a rather quick kludge, based on an 
		iterative model of ballistic travel, and a 
		penetration model based on tensile strength of steel,
		originally worked out for a space weapons sim.

		(I hope to post that on my site sometime soon.)


		Original estimated penetration formula -

		1 kg moving at 20 km/s = 95.24 lbs TNT equivalent energy,
		requires 96.11 cm (37.84") radius of 5 cm (2") steel armor to stop it
		(assuming 200,000 psi ultimate strength; 238,000 lbs force later 
		inferred from 16"/50 penetration) 


		Error levels: ratio of my penetration estimates 
				  to values given in penetration table,
				  for US 16"/50 with super-heavy shell, 
				  before finagle factor incorporated:

			range		    side armor  deck armor

			point blank  - 	1.000		 n/a
			 5,000 yards - 	0.981		0.657
			10,000 yards - 	0.966		0.684
			15,000 yards - 	0.958		0.763
			20,000 yards - 	0.959		0.844
			25,000 yards - 	0.970		0.915
			30,000 yards - 	0.987		0.971
			35,000 yards - 	1.007*	1.006
			40,000 yards - 	1.002		1.024
			42,345 yards - 	1.023		1.022
						
		* The side armor penetration value for 35,000 yards 
		  is anomalous, 'off the curve,' suggesting that 
		  the value in the table may be in error.

		All side armor penetration values are within 5 pct 
		of table figures; all deck penetration values 
		beyond 20,000 yards are within 5 pct of table figures. 

		Note: above calculations were done before normalizing 
			parameters, and may be very slightly different 
			from final results. 

	*****/


	#include <stdio.h>
	#include <conio.h>
	#include <bios.h>
	#include <stdlib.h>
	#include <float.h>
	#include <math.h>
	#include <ctype.h>

	#define  TRUE    1
	#define  FALSE   0


float

	caliber,
	inclination,
	muzzlevel,
	energy, 
	relenergy,
	length,

	elevation,
	angle, 
	advance, 
	travel, 
	climb, 
	altitude, 
	prevalt,
	dragforce, 
	thrust, 
	vel, 
	kinetic, 
	drag, 
	decel,
	weight, 
	dragarea,

	thickness,
	armorquality, 

	shellform;

int     
	count, 
	peaked;

char
	setlength,
	screenreport;



main()
{
	char	test;

	printf ("\n");
	printf ("          BIG GUN \n\n");
	printf ("     Naval Gun Simulator, beta v. 1.1 \n\n");

	printf ("     Copyright 2001, by Rick Robinson \n");
	printf ("          Lyonesse@compuserve.com \n\n\n");


	design();

	for (;;)
		/* endless loop, exit only by "break" */
	{
		do 
		{
			screenreport = TRUE;
			cycle();

			printf ("To save gun data to a file, enter 'y' \n");
			printf ("     (Any other to continue): ");
			if ((test = getch()) == 'y' || test == 'Y')
			{
				screenreport = FALSE;
				writegun();
				test = 'x';
			}
			
			printf ("\n\n");

			printf ("To change gun characteristics, ");
			printf ("or exit program, enter 'x' \n");
			printf ("     (Any other to continue with same gun): ");
		}
		while ((test = getch()) != 'x' && test != 'X');
		printf ("\n\n");

		printf ("     To quit, enter 'x' (any other to proceed): ");
		if ((test = getch()) == 'x' || test == 'X')
		{
			break;
		}
		printf ("\n\n");

		printf ("     To adjust shellform factor only, enter 'y': ");
		if ((test = getch()) == 'y' || test == 'Y')
		{
			printf ("\n\n");
			printf ("     Enter new shellform factor: ");
			scanf ("%f", &shellform);
			shellform = 1.0 / shellform;
			printf ("\n\n");
		}
		else
		{
			printf ("\n\n");
			design();
		}
	}
	printf ("\n\n");

	printf ("");
		/* Spanish inverted exclamation point */
	printf ("Adios!\n\n");
}

/*****          *****/

cycle()
{
	float   
		accel, 
		acceltime, 
		decelerate(),
		penetration();

	char	testchar;


	count = 0;
	travel = 0;
	altitude = 0;
	peaked = FALSE;


	vel = muzzlevel;

	if (screenreport)
	{
		printf ("\n\n\n\n");

		printf ("Enter elevation (deg): ");
		scanf ("%f", &elevation);
		printf ("\n");
	}

	angle = elevation;

	angle = angle / 57.3; /* radian */

	climb = sin (angle);
	advance = cos (angle);

	if (screenreport)
	{
		printf ("    Time    Deceleration   Velocity    ");
		printf ("Flight Angle    Altitude     Range \n\n");
	}


	do 
	{
		decelerate();
	}
	while (altitude > 0);


	if (!screenreport)
		/* DEBUG to escape interactive sequence */
	{
		return;
	}


	printf ("\n\n");

	if ((travel * 1.094) > 50.0)
	{
		printf ("          ");
		printf ("At %4.2f degrees elevation = ", elevation);
		printf ("%3.0f00 yards range - \n\n", ((travel * 1.094) / 100.0));
	}
	else
	{
		printf ("          ");
		printf ("At %4.2f deg elevation = zero yards range - \n\n", elevation);
	}

	printf ("     Flight time     = %6.1f  seconds \n", (count / 10.0));
	printf ("     Impact velocity = %4.0f    feet per second \n", (vel * 3.28));
	printf ("     Angle of fall   = %7.2f degrees \n\n\n", (angle * 57.3));


	printf ("For inclined belt penetration, enter 'y' \n");
	printf ("     (Any other for vertical belt): ");
	if ((testchar = getch()) == 'y' || testchar == 'Y')
		/* inclined belt */
	{
		printf ("\n\n");

		printf ("Enter inclination of target belt (deg): ");
		scanf ("%f", &inclination);
		printf ("\n\n\n");

		inclination /= 57.3;
	}
	else	/* conventional vertical belt */
	{
		printf ("\n\n\n\n");

		inclination = 0.0;
	}

	printf ("     Stopping power of armor varies by period, \n");
	printf ("     and also on armor-shell interaction. \n\n");

	printf ("     Typical values (= relative velocity to penetrate): \n\n");

	printf ("          Wrought iron (1850s-1880s):   0.55-0.60 \n");
	printf ("          Mild steel (1870s-1890s):     0.70-0.75 \n\n");

	printf ("          Compound (1870s-90s):         0.75 \n");
	printf ("          Harvey (1890s):               0.80 \n\n");

	printf ("          Face-hardened, WW I era:      0.83 \n");
	printf ("               (if shell is not decapped) \n\n");
	printf ("          Face-hardened, WW I era:      1.10+ \n");
	printf ("               (if shell is decapped) \n\n");
	printf ("          WW II era:                    1.00 \n\n\n");

	printf ("Enter relative armor quality: ");
	scanf ("%f", &armorquality);
	printf ("\n\n\n");

	if ((travel * 1.094) > 50.0)
	{
		printf ("          ");
		printf ("At %4.2f degrees elevation = ", elevation);
		printf ("%3.0f00 yards range - \n\n", ((travel * 1.094) / 100.0));
	}
	else
	{
		printf ("          ");
		printf ("At %4.2f deg elevation = zero yards range - \n\n", elevation);
	}

	
	printf ("     Belt penetration, ");
	thickness = penetration (vel, 
					 (angle + inclination),
					 weight, 
					 caliber);

	printf ("%5.2f inches \n", (thickness / 2.54));

	angle = 1.571 - angle;

	printf ("     Deck penetration, ");
	thickness = penetration (vel, 
					 angle,
					 weight, 
					 caliber);

	printf ("%5.2f inches \n", (thickness / 2.54));

	printf ("\n\n");

}

/*****          *****/

float decelerate()

	{
	float startadv, startclmb, ratio;

	++count;

	startadv = advance = (vel / 10) * cos (angle);
	startclmb = climb = (vel / 10) * sin (angle);

	if (!peaked)
		{
		climb = climb - 0.098;
		}
	else
		{
		climb = climb + 0.098;
		}

	if (climb <= 0)
		{
		peaked = TRUE;
		climb = fabs (climb);
		}

	vel = 10 * sqrt ((advance * advance) + (climb * climb));
	kinetic = (vel * vel) / 2.0;
	dragforce = kinetic * (1.225 / 4.45) / 10.75;
		/* lbs per sq ft */

	dragforce *= 1.0 / pow (2.718, (altitude / 12000.0));
		/* finagle factor: reduced air density and drag with altitude, 
		   but the density reduction rate is much less than shown;
		   cause of the discrepancy has not been determined  */

	drag = dragforce * dragarea;
	drag *= shellform;
		/* adjust for shell ballistic qualities */

	decel = (drag  / weight) * 9.8;
	vel = vel - (decel / 10);

	angle = atan (climb / advance);

	advance = (vel / 10) * cos (angle);
	climb = (vel / 10) * sin (angle);

	if (!peaked)
		{
		altitude = altitude + ((startclmb + climb) / 2.0);
		}
	else
		{
		altitude = altitude - ((startclmb + climb) / 2.0);
		}

	if (altitude > 0)
		{
		prevalt = altitude;
		travel = travel + ((startadv + advance) / 2.0);
		}
	else /* impact */
		{
		ratio = prevalt / (prevalt + fabs (altitude));
		advance = ratio * ((startadv + advance) / 2.0);
		travel = travel + advance;
		altitude = 0;
		}

	if (screenreport && ((count % 50 == 0) || (altitude == 0)))
		{
		printf (" ");
		printf ("%5.1f sec: ", (count / 10.0));

		printf ("%4.1f gees;   ", (decel / 9.8));
		printf ("%4.0f ft/sec;   ", (vel * 3.28));
		if (!peaked)
			{
			printf ("%5.1f deg;   ", (angle * 57.3));
			}
		else
			{
			printf ("%5.1f deg;   ", (-angle * 57.3));
			}
		if ((altitude * 3.28) >= 50.0)
			{
			printf ("%3.0f00 ft up;   ", ((altitude * 3.28) / 100.0));
			}
		else
			{
			printf ("    0 ft up;   ");
			}

		if ((travel * 1.094) > 50.0)
			{
			printf ("%3.0f00 yds \n", ((travel * 1.094) / 100.0));
			}
		else
			{
			printf (" zero yds \n");
			}
		}
	}

/*****          *****/


design()
	/* gun characteristics */
{
	char testchar;

	do
	{
		testgun();

		printf ("If design is acceptable, enter 'y' \n");
		printf ("     (Any other for new gun stats): ");
	}
	while ((testchar = getch()) != 'y' && testchar != 'Y');

}

/*****          *****/


testgun() 
	/* basic gun stats */
	
{
	float foottons;

	char testchar;


	printf ("\n\n");

	printf ("Enter caliber of gun (inches): ");
	scanf ("%f", &caliber);
	printf ("\n");
	
	dragarea = 0.45 * pow ((caliber / 16.0), 2.0);
		/* drag area proportional to 
		   square of caliber */


	printf ("     Typical range of shell weights: \n\n");

	printf ("          Round shot       - ");
	printf ("%1.0f lbs \n", (pow (caliber, 3.0) / 8.0));

	printf ("          Early rifled gun - ");
	printf ("%1.0f lbs \n", (pow (caliber, 3.0) / 3.5));

	printf ("          1880s            - ");
	printf ("%1.0f lbs \n\n", (pow (caliber, 3.0) / 2.42));


	printf ("          World War I      - ");
	printf ("%1.0f lbs \n", (pow (caliber, 3.0) / 2.0));

	printf ("          World War II     - ");
	printf ("%1.0f lbs \n", (pow (caliber, 3.0) / 1.85));

	printf ("          US 'super-heavy' - ");
	printf ("%1.0f lbs \n\n", (pow (caliber, 3.0) / 1.517));
	
	printf ("Enter weight of shell: ");
	scanf ("%f", &weight);
	printf ("\n\n\n");


	if (weight >= (pow (caliber, 3.0) / 3.0))
	{
		printf ("You may use gun length (in calibers) ");
		printf ("to determine muzzle velocity -- \n\n");

		printf ("To specify gun length in calibers, type 'y' \n");
		printf ("     (Any other to enter muzzle velocity): ");
	
		if ((testchar = getch()) == 'y' || testchar == 'Y')
		{
			setlength = TRUE;
			printf ("\n\n");
		}
		else
		{
			setlength = FALSE;
			printf ("\n\n");
		}
	}
	else
	{
		setlength = FALSE;
		
		printf ("\n\n");
		printf ("     Note: Sim model understates length of early guns; \n");
		printf ("           gun length specification model not offered. \n\n");
	}

	if (setlength)
	{
		printf ("Enter length in calibers: ");
		scanf ("%f", &length);
		printf ("\n");

		relenergy = pow ((length / 50.0), 1.333);

		energy = relenergy * (pow (caliber, 3.0) / 11.51);
			/* in megajoules */

		muzzlevel = sqrt ((2.0 * (energy * 1.0e6)) / (weight / 2.204));
			/* meters per second */

		printf ("     Muzzle velocity = ");
		printf ("%1.0f fps \n\n\n", (muzzlevel * 3.28));
	}
	else
	{
		printf ("     Typical muzzle velocities: \n\n");

		printf ("          1860s-70s:      1100-1400 fps \n");
		printf ("          1880s:          1900 fps \n\n");

		printf ("          20th c. 'slow': 2400 fps \n");
		printf ("          20th c. 'fast': 2850 fps \n\n");
		printf ("     (Some excellent guns had 'slow' muzzle velocities) \n\n");

		printf ("Enter muzzle velocity (feet per second): ");
		scanf ("%f", &muzzlevel);
		printf ("\n\n");
		muzzlevel /= 3.28;
			/* convert to meters/sec */	


		energy = ((weight / 2.204) * pow (muzzlevel, 2.0)) / 2.0;
			/* muzzle energy in joules */
		energy /= 1.0e6;
			/* megajoules */

		relenergy = energy * (11.51 / pow (caliber, 3.0));
	}

	printf ("     ");
	if (energy > 100.0)
		{
		printf ("Muzzle energy = %3.1f megajoules = ", energy);
		}
	else
		{
		printf ("Muzzle energy = %5.3f megajoules = ", energy);
		}
	
	foottons = energy * 1.0e6;
		/* joules again */
	foottons *= (0.737 / 2000.0);
		/* foot-tons */

	if (foottons > 100.0)
		{
		printf ("%3.1f foot-tons \n\n", foottons);
		}
	else
		{
		printf ("%5.3f foot-tons \n\n", foottons);
		}


	printf ("     Relative muzzle energy and typical barrel length: \n\n");

	printf ("          Early guns (1860s):         about 0.05-0.15 \n");
	printf ("               (Program understates gun lengths) \n\n");

	printf ("          25 caliber gun (1880s):     about 0.3-0.4 \n");
	printf ("          WW I era 45 caliber gun:    about 0.8-0.9 \n\n");

	printf ("          US 16\"/50, superheavy shell:      1.0 \n");
	printf ("          German WW II 11\"/54.5:            1.13 \n\n");

	printf ("     Relative muzzle energy: %4.2f \n\n", relenergy);

	if (!setlength)
	{
		length = pow (relenergy, 0.75) * 50.0;

		printf ("     Estimated length: %1.0f calibers \n\n\n", length);
	}
	else
	{
		printf ("\n\n");
	}


	printf ("Type any character to continue: ");
	getch();
	printf ("\n\n\n");


	printf ("     Shellform factor adjusts for ");
	printf ("ballistic performance of shells \n\n");

	printf ("          Typical value ranges: \n\n");

	printf ("          19th century:       0.50-0.55 \n");
	printf ("          WW I:               0.60-0.80 \n\n");
	printf ("          WW II:        about 1.0 \n");
	printf ("          WW II (Germany):    1.1 \n\n");

	printf ("Enter shellform factor: ");
	scanf ("%f", &shellform);
	printf ("\n\n");
	shellform = 1.0 / shellform;

} /* end */

     /********************/


density()
	/* atmospheric density with altitude;
	   test function; formula placed inline */
{
	float
		altitude,
		airdensity;

	printf ("Enter altitude: ");
	scanf ("%f", &altitude);
	printf ("\n");

	airdensity = 1.0 / pow (2.718, (altitude / 33000.0));

	printf ("     Value is %6.4f \n\n", airdensity);
	
}

	/********************/

float penetration (velocity, 
		 angle,
		 mass, 
		 radius)

	/* armor penetration */

	float
		velocity,
		angle,
		mass,
		radius;

{
	float
		cosine,		/* of impact angle */
		armorrad;		/* radius of armor area absorbing impact */


	char	testchar;		/* user selection test */


	mass /= 2.204;
		/* convert to kg */

	radius /= 2.0;
		/* convert from diameter to radius */

	radius *= 2.54;
		/* convert to cm */

	velocity /= 1000.0;
		/* convert to km/sec */

	velocity /= armorquality;
		/* adjust for armor quality 
		   (defined in terms of impact velocity) */

	cosine = cos (angle);

	if ((angle < 1.56) && (angle > 1.15))
		/* finagle factor for shell 'nosing in';
		   multiply cosine of impact angle 
		   (= velocity ratio) by 1.4 at 90 deg,
		   decreasing linearly to zero at 66 deg */ 
	{
		cosine *= 1.0 + ((angle - 1.15) / 1.04);
	}

	angle = acos (cosine);

	velocity *= cos (angle);
		/* inward component; 
		   straight-on impact has zero angle, 
		   full effective velocity */

		/* check above for belt v deck! */

	armorrad = sqrt (mass) * velocity; 
		/* relative to constant */

	armorrad *= 9.852;
		/* estimated constant value */

	thickness = pow ((armorrad / radius), 2.0);

	armorrad = radius;
		/* linear plug of armor facing shell */

	thickness *= pow ((radius / thickness), 0.3333);

	if (thickness >= radius)
		/* DEBUG check this! */
	{
		armorrad *= pow ((thickness / radius), 0.25);
			/* allows for 'spread' of force as it is
			   transmitted through armor thickness */
	}

	return (thickness);


}

	/***************/


writegun()
	/* write stats to file */
{

	float 
		foottons,

		belt, 
		deck,
		chkbelt,
		chkdeck,

		maxrange,
		maxelev,

		penetration();

	FILE    *fopen(), *gunreport;
		/* file saving vars */


	char    gunmark[50], filename[20];
		/* type and name, filename */


	printf ("\n\n");
	printf ("Enter file name for gun type: ");
	scanf ("%s", filename);
	gunreport = fopen (filename, "w");
		/* open file */

	printf ("\n");

	flushall();
	printf ("Enter gun mark or designation: ");
	gets (gunmark);

	fprintf (gunreport, "\n");
	fprintf (gunreport, "          %s \n\n\n", gunmark);

	fprintf (gunreport, "     Caliber = %3.1f inch ", caliber);
	fprintf (gunreport, "(%3.1f cm) \n", (caliber * 2.54));

	fprintf (gunreport, "     Shell weight = %1.0f lbs ", weight);
	fprintf (gunreport, "(%1.0f kg) \n", (weight / 2.204));

	fprintf (gunreport, "     Muzzle velocity = ");
	fprintf (gunreport, "%1.0f fps ", (muzzlevel * 3.28));
	fprintf (gunreport, "(%1.0f m/s) \n\n", muzzlevel);

	fprintf (gunreport, "     Relative ballistic performance: ");
	fprintf (gunreport, "%4.2f \n\n", 1.0 / shellform);


	fprintf (gunreport, "     ");
	if (energy > 100.0)
		{
		fprintf (gunreport, "Muzzle energy = %3.1f megajoules = ", energy);
		}
	else
		{
		fprintf (gunreport, "Muzzle energy = %5.3f megajoules = ", energy);
		}
	
	foottons = energy * 1.0e6;
		/* joules again */
	foottons *= (0.737 / 2000.0);
		/* foot-tons */

	if (foottons > 100.0)
		{
		fprintf (gunreport, "%3.1f foot-tons \n\n", foottons);
		}
	else
		{
		fprintf (gunreport, "%5.3f foot-tons \n\n", foottons);
		}

	fprintf (gunreport, "     Relative muzzle energy: ");
	fprintf (gunreport, "%4.2f \n\n", relenergy);


	if (weight >= (pow (caliber, 3.0) / 3.0))
	{
		if (setlength)
		{
			fprintf (gunreport, "     Barrel length: ");
		}
		else
		{
			fprintf (gunreport, "     Typical barrel length: ");
		}
		fprintf (gunreport, "%1.0f calibers \n\n\n", length);
	}
	else
	{
		fprintf (gunreport, "\n");
	}


	fprintf (gunreport, "     Elevation          Range");
	fprintf (gunreport, "        Time      Velocity   Fall Angle \n\n");

	elevation = 2.5;

	do 
	{
		cycle();

		if ((travel * 1.094) > 50.0)
		{
			fprintf (gunreport, "     %5.1f deg", elevation);
			fprintf (gunreport, "      ");
			fprintf (gunreport, "%4.0f00 ", ((travel * 1.094) / 100.0));
			fprintf (gunreport, "yards");

			fprintf (gunreport, "   %5.1f sec", (count / 10.0));
			fprintf (gunreport, "   %4.0f fps", (vel * 3.28));
			fprintf (gunreport, "   %5.1f deg \n", (angle * 57.3));
		}

		if (elevation <= 14.99)
		{
			elevation += 2.5;
		}
		else
		{
			elevation += 5.0;
		}


	}
	while (elevation <= 50.0);


	thickness = penetration (muzzlevel, 
					 inclination,
					 weight, 
					 caliber);

	fprintf (gunreport, "\n\n");


	fprintf (gunreport, "          ");
	fprintf (gunreport, "Armor Penetration - ");
	if (inclination)
	{
		fprintf (gunreport, "Belt Inclined ");
		fprintf (gunreport, "%1.0f degrees \n\n", (inclination * 57.3));
	}
	else
	{
		fprintf (gunreport, "Vertical Belt Armor \n\n");
	}	

	fprintf (gunreport, "               ");
	fprintf (gunreport, "(Relative armor quality, %4.2f) \n\n", armorquality);

	fprintf (gunreport, "               ");
	fprintf (gunreport, "Maximum penetration: %4.2f inches \n\n\n", (thickness / 2.54)); 	


	elevation = 1.0;
	cycle();
	chkbelt = penetration (vel, 
				     (angle + inclination),
				     weight, 
				     caliber);

	belt = 0.0;
	deck = 2.54;
	maxrange = 0.0;

	while ((belt + 5.08) <= thickness)
	{
		belt += 5.08;
	}

	
	fprintf (gunreport, "     Elevation          Range");
	fprintf (gunreport, "            Belt       Deck \n\n");


	elevation = 1.0025;

	do 
	{
		cycle();

		if (travel > maxrange)
		{
			maxrange = travel;
			maxelev = elevation;
		}

		chkbelt = penetration (vel, 
					     (angle + inclination),
					     weight, 
					     caliber);


		chkdeck = penetration (vel, 
					     (1.571 - angle),
					     weight, 
					     caliber);


		if ((chkbelt <= belt) || (chkdeck >= deck))
		{
			fprintf (gunreport, "     %5.1f deg", elevation);
			fprintf (gunreport, "      ");
			fprintf (gunreport, "%4.0f00 ", ((travel * 1.094) / 100.0));
			fprintf (gunreport, "yards        ");

			if (chkbelt <= belt)
			{
				fprintf (gunreport, "%3.0f in", (chkbelt / 2.54));
				belt -= 5.08;

				if (chkdeck >= deck)
				{
					fprintf (gunreport, "     ");
					fprintf (gunreport, "%3.0f in \n", (chkdeck / 2.54));
					deck += 2.54;
				}
				else 
				{
					fprintf (gunreport, "\n");
				}
			}
			else 
			{
				if (chkdeck >= deck)
				{
					fprintf (gunreport, "  ...      ");
					fprintf (gunreport, "%3.0f in \n", (chkdeck / 2.54));
					deck += 2.54;
				}
			}
		}

		elevation *= 1.0025;


		printf (".");
	}
	while (elevation <= 50.0);

	fprintf (gunreport, "\n\n");
	fprintf (gunreport, "     ");
	fprintf (gunreport, "Maximum range = ");
	fprintf (gunreport, "%1.0f00 ", ((maxrange * 1.094) / 100.0));
	fprintf (gunreport, "yards at %3.1f deg elevation \n\n\n\n", maxelev);

	fclose (gunreport);
		/* close file */

} /* end */

