/* Diurnal Simulator v. 1.1 (6/18/99) */

	/* Copyright 1999 by Rick Robinson */

	#include <stdio.h>
	#include <conio.h>
	#include <bios.h>
	#include <stdlib.h>
	#include <float.h>
	#include <math.h>
	#include <ctype.h>


	#define TRUE            1
	#define FALSE           0


	#define PI              3.1416
	#define RADIAN         57.3

	#define ARC            90

	#define HOURS          24

	#define CELSIUS       273.0

	#define SOLCONST      395.6
		/* radiative balance temperature
		   for vertical sun at 1 AU */

	#define EARTHTEMP     288.5
		/* average temperature of Earth, K */

	#define HEATRATE      1.17
		/* ocean surface heating (deg K)
		   from one hour of vertical
		   solar radiation; applies to
		   top meter of ocean surface */

	#define SNOWTEMP     273.0
		/* maximum persisting snow-cover
		   temperature: 0 C */

	#define ICETEMP      260.0
		/* maximum sea ice
		   temperature: -13 C */

	#define ICEALBEDO      0.6
		/* additional reflectivity
		   due to ice or snow surface
		   (albedo of ice or snow
		   itself is up to 0.9;
		   reduced value due to
		   atmospheric absorption, etc.) */


float   luminosity, orbitdist, albedo,
	declination, latitude,
	ocean, relcont, landsurf,
	hoursun, totalsun,
	temperature, greenheat,
	soltemp, basetemp, latavgtemp,
	increment, heatstore,
	avgtemp, avgradtemp, avggrnht;


int     degree, day, hour, daylength, run;

FILE	*fopen(), *planet;

char    heatsink, world[20], planetname[50];


main()
	{
	char test;


	printf ("\n");
	printf ("Daily Temperature Simulator, v. 1.1 ");
	printf ("\n\n");
	printf ("     ");
	printf ("Copyright 1999 by Rick Robinson \n");
	printf ("     ");
	printf ("All rights reserved \n\n");
	printf ("     ");
	printf ("(e-mail: Lyonesse@compuserve.com) ");


	do {
		printf ("\n\n");
		setup();
		printf ("To quit, enter 'X' ");
		printf ("(any other to continue): ");
		}
	while ((test = getch()) != 'x' && test != 'X');
	printf ("\n\n");
	printf ("\xad");
	printf ("Adios!\n\n");
	}

/*****          *****/

setup()
	/* global parameters and latitude */

	{
	float setsun, settemp;
	char newlocal;


	makefile();

	printf ("Enter luminosity of star (Sun = 1.0): ");
	scanf ("%f", &luminosity);
	fprintf (planet, "     ");
	fprintf (planet, "Stellar luminosity, %3.2f Suns \n", luminosity);
	printf ("\n");

	printf ("Enter orbital distance in AU: ");
	scanf ("%f", &orbitdist);
	fprintf (planet, "     ");
	fprintf (planet, "Orbital distance, %4.3f AU \n\n", orbitdist);
	printf ("\n\n");


	printf ("Enter ocean surface of planet (percent): ");
	scanf ("%f", &ocean);
	fprintf (planet, "     ");
	fprintf (planet, "Ocean surface of planet, ");
	fprintf (planet, "%1.0f percent \n", ocean);
	printf ("\n");

	relcont = 1.0 - (ocean / 100.0);
		/* convert from sea percentage to land fraction */

	relcont *= 0.93;
	relcont = pow (relcont, 4.0) / 0.748;
	relcont *= 100.0;
		/* average continentality of planet */

	fprintf (planet, "     ");
	fprintf (planet, "Continentality of planetary average climate, ");
	fprintf (planet, "%2.1f percent \n\n", relcont);

	printf ("     ");
	printf ("Continentality of planetary average climate, ");
	printf ("%2.1f percent \n\n", relcont);


	printf ("Enter latitude: ");
	scanf ("%i", &degree);
	fprintf (planet, "     ");
	fprintf (planet, "Latitude %1i degrees \n", degree);
	printf ("\n");

	printf ("Enter seasonal declination of sun: ");
	scanf ("%f", &declination);
	fprintf (planet, "     ");
	fprintf (planet, "Declination of sun, %3.2f deg \n", declination);
	printf ("\n");


	printf ("Enter average temperature for ");
	printf ("latitude and season (deg F): ");
	scanf ("%f", &latavgtemp);
	fprintf (planet, "     ");
	fprintf (planet, "Zonal average temperature, ");
	fprintf (planet, "%3.2f F \n\n", latavgtemp);

	latavgtemp = ((latavgtemp - 32.0) * (5.0 / 9.0)) + 273.0;
	basetemp = latavgtemp;

	printf ("\n");

	printf ("Enter zonal greenhouse heating (deg K): ");
	scanf ("%f", &greenheat);
	fprintf (planet, "     ");
	fprintf (planet, "Zonal greenhouse heating, ");
	fprintf (planet, "%3.2f K \n\n", greenheat);
	printf ("\n");

	newlocal = FALSE;

	do {
		localsetup (newlocal);

		fclose (planet);
		printf ("\n\n");

		printf ("To exit, or change global values ");
		printf ("(e.g., sun and latitude), \n");
		printf ("    ");
		printf ("enter 'X' (any other to proceed): ");

		if ((newlocal = getch()) != 'x' && newlocal != 'X')
			{
			newlocal = TRUE;
			printf ("P \n\n");
			makefile();

			fprintf (planet, "     ");
			fprintf (planet, "Stellar luminosity, ");
			fprintf (planet, "%3.2f Suns \n", luminosity);

			fprintf (planet, "     ");
			fprintf (planet, "Orbital distance, ");
			fprintf (planet, "%4.3f AU \n\n", orbitdist);

			fprintf (planet, "     ");
			fprintf (planet, "Ocean surface of planet, ");
			fprintf (planet, "%1.0f percent \n", ocean);

			fprintf (planet, "     ");
			fprintf (planet, "Continentality of ");
			fprintf (planet, "planetary average climate, ");
			fprintf (planet, "%2.1f percent \n\n", relcont);



			fprintf (planet, "     ");
			fprintf (planet, "Latitude ");
			fprintf (planet, "%1i degrees \n", degree);

			fprintf (planet, "     ");
			fprintf (planet, "Declination of sun, ");
			fprintf (planet, "%3.2f deg \n", declination);

			fprintf (planet, "     ");
			fprintf (planet, "Zonal average temperature, ");
			fprintf (planet, "%3.2f F \n\n", (((latavgtemp -
							 273.0) *
							 (9.0 / 5.0)) +
							 32.0));

			fprintf (planet, "     ");
			fprintf (planet, "Zonal greenhouse heating, ");
			fprintf (planet, "%3.2f K \n\n", greenheat);
			}
		else
			{
			newlocal = FALSE;
			printf ("X \n\n");
			}
		}
	while (newlocal);

	printf ("\n\n");
	}

/*****     *****/

localsetup (repeat)
	char repeat;

	/* local characteristics, day length,
	   and background temperature */
	{
	float fahren;

	if (repeat)
		{
		printf ("          ");
		printf ("Continentality of climate reflects \n");
		printf ("     ");
		printf ("rapid decrease of ocean's moderating effect \n");
		printf ("     ");
		printf ("as one moves inland. \n\n");

		printf ("          ");
		printf ("Zero pct = maritime climate \n");
		printf ("          ");
		printf ("  10 pct = coastal climate \n");
		printf ("          ");
		printf ("  50 pct = moderate inland climate \n");
		printf ("          ");
		printf ("  90 pct = severe continental climate \n");
		printf ("          ");
		printf (" 100 pct = extreme climate \n\n");

		printf ("     ");
		printf ("(Continentality of planetary average, ");
		printf ("%2.1f percent) \n\n", relcont);

		printf ("Enter continentality of climate, ");
		printf ("as percentage: ");

		scanf ("%f", &landsurf);

		fprintf (planet, "     ");
		fprintf (planet, "Continentality of climate, ");
		fprintf (planet, "%3.1f percent \n", landsurf);

		printf ("\n");
		}
	else
		landsurf = relcont;

	landsurf *= 0.01;

	landsurf *= 0.748; /* ref 0.75 */

	landsurf = pow (landsurf, 0.25); /* ref 0.25 */
		/* emperical rule, "continentality"
		   is proportional to fourth root of
		   reduction in oceanic heat sink effect */

	printf ("\n");

	if (repeat)
		{
		printf ("Enter regional albedo ");
		printf ("(clear land 15-30 percent; \n");
		printf ("     ");
		printf ("clear ocean 10 pct; ");
		printf ("full overcast 30-90 pct; \n");
		printf ("     ");
		printf ("snow or icefields 60-90 pct; \n");
		printf ("     ");
		printf ("Earth nonpolar average, 29 pct): ");
		scanf ("%f", &albedo);
		albedo *= 0.01;
		}
	else
		{
		printf ("Enter planetary nonpolar albedo, ");
		printf ("as percentage \n");
		printf ("     ");
		printf ("(program will adjust for polar regions): ");
		scanf ("%f", &albedo);
		albedo *= 0.01;

		if (latavgtemp < SNOWTEMP)
			/* effects of snow/ice cover
			   on albedo */
			{
			if (latavgtemp < ICETEMP)
				{
				if (ICEALBEDO > albedo)
					{
					albedo = ICEALBEDO;
					}
				}
			else
				{
				if (((1.0 - ocean) * ICEALBEDO) > albedo)
					{
					albedo = (1.0 - ocean) * ICEALBEDO;
					}
				}
			}
		}


	fprintf (planet, "     ");
	fprintf (planet, "Regional albedo, %3.2f \n\n", albedo);
	printf ("\n");


	printf ("Enter length of planetary day, in hours: ");
	scanf ("%i", &daylength);
	fprintf (planet, "     ");
	fprintf (planet, "Length of day, %1i hours \n\n", daylength);
	printf ("\n");

	heatstore = HEATRATE *
		    (1.0 / pow (((float) daylength / 24.0), 0.25));

		/* emperical value for heat transfer
		   "averaging" effect during long day */


	printf ("Enter run length to equilibrate (typically 5-20 days): ");
	scanf ("%i", &run);

	temperature = basetemp;
	increment = 0.0;
	avgradtemp = basetemp * 1.41;

	printf ("\n\n");

	heatsink = TRUE;

	if (!repeat)
		{
		for (;;)
			{
			cycle();

			printf ("\n\n");
			printf ("++++++++++++++++++++++++++++ \n\n");

			fprintf (planet, "++++++++++++++++++++++++++++ ");
			fprintf (planet, "\n\n");

			if (fabs ((avgtemp - latavgtemp) > 0.005))
				basetemp -= avgtemp - latavgtemp;
			else
				break;
			}
		}
		/* adjust stabilizing temperature till it
		   produces the latitude average temperature */


	printf ("\n\n");
	printf ("          ++++++++++++++++++++++++++++++++++++++ \n\n");


	if (repeat)
		{
		heatsink = TRUE;
		cycle();
		}

	fclose (planet);
	printf ("\n\n");
	}

/*****     *****/

cycle()
	{
	for (day = 1; day <= run; day++)
		{
		printf ("Day %1i: \n\n", day);

		fprintf (planet, "          ", day);
		fprintf (planet, "==================================");
		fprintf (planet, "\n\n");
		fprintf (planet, "                       ");
		fprintf (planet, "Day %1i \n", day);

		sunlight();

		fprintf (planet, "\n\n\n");
		}
	}

/*****     *****/

makefile()
	{
	flushall();

	printf ("\n\n");
	printf ("Enter header note: ");
	gets (planetname);
	printf ("\n");

	printf ("Enter filename: ");
	scanf ("%s", world);
	planet = fopen (world, "w");
	printf ("\n");

	fprintf (planet, "\n");
	fprintf (planet, "          ");
	fprintf (planet, "%s", planetname);
	fprintf (planet, "\n\n\n");
	}

/*****     *****/

sunlight()
	/* much better model for
	   daily sunlight at a given latitude
	   and solar declination */

	{
	float   zenith, nadir,
		highsun, lowsun, midsun,
		radius, rotation,
		sunheight, sunslope,
		insolation, dayturn, netrad, chkrad,
		high, low, fahren, radtemp, tempchange;


	zenith = (ARC - degree) + declination;
	if (zenith > ARC)
		zenith = (2.0 * ARC) - zenith;

	highsun = sin (zenith / RADIAN);

	nadir = -((ARC - degree) - declination);
	if (nadir < -ARC)
		{
		nadir += 2.0 * ARC;
		nadir = -nadir;
		}

	lowsun = sin (nadir / RADIAN);

	midsun = (highsun + lowsun) / 2.0;
	radius = (highsun - lowsun ) / 2.0;


	fprintf (planet, "\n");
	printf ("\n");

	insolation = 0.0;
	high = 0.0;
	low = 1000.0;
	avgtemp = 0.0;

	for (hour = 1; hour <= daylength; hour++)
		{
		dayturn = ((float) hour / daylength);
		rotation = ((dayturn * 360.0) - 90.0) / RADIAN;
		if (((sin (rotation) * radius) + midsun) > 0.0)
			{
			hoursun = (sin (rotation) * radius) +
				  midsun;
			sunheight = asin (hoursun) * RADIAN;

			hoursun *= pow ((1.0 / orbitdist), 2.0) *
				luminosity;

			insolation += hoursun;
			}
		else
			hoursun = sunheight = 0.0;


		netrad = (hoursun * (1.0 - albedo)) -
			 pow (((temperature - greenheat) /
			 SOLCONST), 4.0);


		tempchange = netrad * (heatstore *
			     (1.0 / (1.0 - landsurf)));

			/* difference between ingoing and
			   outgoing radiation, times a
			   constant (times interval) */

		chkrad = (hoursun * (1.0 - albedo)) -
			 pow ((((temperature + tempchange) - greenheat) /
			 SOLCONST), 4.0);

		chkrad = chkrad * (heatstore *
			 (1.0 / (1.0 - landsurf)));

		chkrad /= tempchange;

		tempchange = tempchange * ((1.0 + chkrad) / 2.0);

		if (temperature < ICETEMP)
			tempchange *= (3.0 - (landsurf * 2.0));
			/* reduced heat sink effect
			   over sea ice causes faster
			   temperature changes */

		radtemp = (pow (hoursun, 0.25) *
			  (SOLCONST * pow ((1.0 - albedo), 0.25))) +
			  greenheat;


		if (((temperature < radtemp) &&
		   ((temperature + tempchange) > radtemp)) ||
		   ((temperature > radtemp) &&
		   ((temperature + tempchange) < radtemp)))
			/* overshot or undershot */
			{
			printf ("\n Bump-up \n");
			fprintf (planet, "\n     Bump-up -- \n");
			temperature = radtemp;
			}
		else
			temperature += tempchange;


		if (heatsink)
			{
			temperature -= (temperature - basetemp) /
				       (50.0 *
				       pow (((float) daylength /
				       24.0), 0.5));
			}
				/* emperical value -- ref 50.0, 0.5 */

				/* simulates heat sink and
				   seasonal "drag" effects
				   of oceans */


		avgtemp += temperature;

		if (temperature > high)
			high = temperature;
		if (temperature < low)
			low = temperature;

		if ((day == run) && ((daylength < 24) ||
				    (hour % (daylength / 24) == 0)))

			{
			fprintf (planet, "Hour %3i:    ", hour);

			fprintf (planet, "Elev: ");
			if (hoursun > 0.0)
				fprintf (planet, "%3.0f deg    ", sunheight);
			else
				fprintf (planet, "   ...     ");

			fprintf (planet, "Insol: ");
			if (hoursun > 0.0)
				fprintf (planet, "%5.2f    ", hoursun);
			else
				fprintf (planet, "%5.2f    ", 0.0);

			fprintf (planet, "Net rad: %5.2f    ", netrad);

			fahren = ((temperature - 273.0) * (9.0 / 5.0)) + 32.0;
			fprintf (planet, "Temp: %6.1f F\n", fahren);

			if ((hour % 24) == 0)
				fprintf (planet, "\n");
			}

		printf (".");
		if ((hour % 24) == 0)
			printf ("\n");


		/* end new block */

		}

	insolation /= daylength;


	avgtemp /= daylength;

	printf ("\n");
	fprintf (planet, "\n");


	fprintf (planet, "Average insolation: %3.2f \n\n", (insolation *
							   (1.0 - albedo)));

	fahren = ((high - 273.0) * (9.0 / 5.0)) + 32.0;
	fprintf (planet, "High temperature: %7.2f F \n", fahren);

	fahren = ((low - 273.0) * (9.0 / 5.0)) + 32.0;
	fprintf (planet, "Low temperature:  %7.2f F \n\n", fahren);

	fahren = ((avgtemp - 273.0) * (9.0 / 5.0)) + 32.0;
	fprintf (planet, "Average temperature: ");
	fprintf (planet, "%5.2f K = %5.2f F \n\n", avgtemp, fahren);

	printf (" Average temperature: %5.2f F \n\n\n", fahren);

	avgradtemp = (pow ((insolation * (1.0 - albedo)), 0.25) * SOLCONST) +
		     greenheat;


	high = (pow (((1.0 - albedo) *
	       pow ((1.0 / orbitdist), 2.0)), 0.25) * SOLCONST) +
	       greenheat;

		/* var high re-used for
		   vertical radiation temperature */

	fahren = ((high - 273.0) * (9.0 / 5.0)) + 32.0;
	fprintf (planet, "Vertical sun balance temperature: ");
	fprintf (planet, "%5.2f K = %5.2f F \n", high, fahren);


	fahren = ((avgradtemp - 273.0) * (9.0 / 5.0)) + 32.0;
	fprintf (planet, "Radiative balance temperature: ");
	fprintf (planet, "%5.2f K = %5.2f F \n", avgradtemp, fahren);


	fprintf (planet, "Effective radiation at avg temp: ");
	fprintf (planet, "%3.2f \n\n", pow (((avgtemp - greenheat) /
				       SOLCONST), 4.0));

	}

/** END **/


                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
