/* Galley Rowing Speed Simulator, beta v. 1.0 (5/5/2001; 8/8/98) */


	/* standard function libs */

	#include <stdio.h>
	#include <conio.h>
	#include <bios.h>
	#include <stdlib.h>
	#include <float.h>
	#include <math.h>


	/* constants */

	#define TRUE    1
	#define FALSE   0

	#define TON    35.0
		/* cubic feet of water per ton */
	#define ROWPWR  6.0
		/* rowers needed for 1 hp at full stroke,
		   for a 20-min dash */
	#define CHPWR   0.75
		/* strength proportion for 2-hr chase */
	#define CRPWR   0.6
		/* strength proportion for 8-hr cruise */
	#define SFPWR   0.2
		/* strength proportion for three shifts */

	/* external variables */

float   length, beam, depth, displacement,
	burden, custburden,
	blockratio, displacement, submergence,
		/* hull attributes */

	cosharp, cowave, cofriction, frict, wave,
	hp, basehp, availhp, speed, dash, chase, cruise, shift,
	rowers, dashrow, chaserow, cruiserow, shiftrow,
	bench, benchfactor;
		/* determination of speed */

FILE    *fopen(), *shiptype;
		/* file saving vars */

char    name[50], filename[20];
		/* type and name, filename */

	/* end declarations */

/***************          ***************/


main()
	/* program chassis */

	{
	char escape;
		/* exit test char */


	printf ("\n\n");
	printf ("Galley Rowing Speed Simulator, beta v. 1.0 \n\n");
	do /* design process */
		{
		cycle(); /* design sequence */
		printf ("\n     ");
		printf ("To exit program, enter 'x'; ");
		printf ("any other to proceed: ");
		}
	while ((escape = getch()) != 'x' && escape != 'X');

	printf ("\n\n");
	printf ("\xad");
		/* Spanish inverted exclamation point */
	printf ("Adios!\n\n");

	} /* end main() */

/***************          ***************/


cycle()
	/* design a ship */

	{
	char escape;
		/* break-off test char */


	/* name of ship or type */

	flushall();
		/* clear char buffer */

	printf ("\n\n     ");
	printf ("Enter ship name or type: ");
	gets (name);
	printf ("\n\n");


	/* hull dimensions and characteristics */

	printf ("     (All dimensions on waterline): \n\n");

	printf ("     Enter length: ");
	scanf ("%f", &length);

 	printf ("     Enter beam: ");
	scanf ("%f", &beam);

	printf ("     Enter depth: ");
	scanf ("%f", &depth);
	printf ("\n\n");

	printf ("Block coefficient gives displacement ");
	printf ("relative to dimensions: \n\n");
	printf ("     0.40 is a very fine, 'hollow' hull \n");
	printf ("     0.52-0.55 for typical galley hulls \n");
	printf ("     0.65 is a full or shallow-draft hull \n\n");
	
	printf ("     Enter block ratio: ");
	scanf ("%f", &blockratio);
	if (blockratio > 0.8)
		/* blockratio above range */
		{
		blockratio = 0.8;
		printf ("Over range; set to 0.8 \n");
		}
	if (blockratio < 0.2)
		/* blockratio below range */
		{
		blockratio = 0.2;
		printf ("Below range; set to 0.2 \n");
		}
	printf ("\n");


	cosharp = 0.375 * pow (((beam / length) / 0.2), 0.2) * 
	          pow ((blockratio / 0.52), 0.333);
		/* emperical hull sharpness formula; 
		   proportional to 5th root of length-beam ratio, 
		   and cube root of block ratio */
		   
	printf ("     Sharpness coefficient = %3.2f ", cosharp);
	if (cosharp < 0.35)
		/* adjust for implication by Rodgers that
		   minimum sharpness coefficient is 0.35 */
		{
		printf ("-- reset to 0.35 minimum value \n\n\n");
		cosharp = 0.35;
		}
	else
		printf ("\n\n\n");
	

	displacement = ((length * beam * depth) / TON) * blockratio;
	printf ("     (Type hull displacement, ");
	printf ("%2.1f tons) \n\n", displacement);


	/* proceed if dimensions and displacement satisfactory;
	   otherwise abort to main() for new dimensions or exit */

	printf ("     To cancel this design, type 'x'; ");
	printf ("any other to proceed: ");
	if ((escape = getch()) != 'x' && escape != 'X')
		/* proceed with design */
		{
		printf ("\n\n");
		design();
		}
	else
		/* abort */
		{
		printf ("\n\n");
		printf ("          Cancelled (dimensions: ");
		printf ("%2.0f x %2.1f x %2.1f; ", length, beam, depth);
		printf ("%3.2f block coefficient) \n\n", blockratio);
		}

	} /* end cycle() */

/***************          ***************/


design()
	/* continue with full design
	   characteristics of ship */

	{
	char newrow;
		/* test character for different 
		   number of rowers */


	printf ("\n");
	printf ("Displacement of %s ", name);
	printf ("is %2.1f tons \n\n", displacement);


	/* hull characteristics determine speed potential */

	cofriction = (length * depth * 1.7) + 
				 ((displacement * TON) / depth);
		/* wetted surface, revised formula, 
		   uses volume in cubic feet */

	cowave = pow (displacement, 0.6667) / length;
	cowave = cowave * cosharp;
		/* wave-making effects */

	submergence = ((length * beam) * sqrt (blockratio)) / TON;
	submergence = submergence / 12.0;
		/* change in waterline per ton of load,
		   converted from feet to inches */

	printf ("          Wave Coefficient is %3.2f \n", cowave);
	printf ("          Wetted Surface is ");
	printf ("%1.0f square feet \n", cofriction);
	cofriction = cofriction / 100.0;
		/* hull friction coefficient based on wetted surface */

	printf ("          ");
	printf ("'M' = %3.2f \n\n", (length / 3.28) /
				     pow (displacement, 0.333));
		/* expresses relative speed capability
		   of hull forms */

	printf ("          ");
	printf ("Submergence %3.2f tons/inch \n\n", submergence);
		/* report */

	burden = ((length - (0.75 * beam)) * beam * depth) / 100.0;
		/* traditional "Old English Carpenters' Measure"
		   of burden; keel length estimated as waterline length
		   less 3/4 beam */

	printf ("          ");
	printf ("%2.1f tons burden, carpenters' measure \n", burden);

	custburden = ((length - (0.6 * beam)) * beam * (beam / 2.0)) / 94.0;
		/* "custom-house" measure of burden */

	printf ("          ");
	printf ("%2.1f tons burden, custom-house measure ", custburden);


	/* determine speed; report, and repeat
	   for different numbers of rowers */

	do
		/* repeat cycle */
		{
		findspeed();
			/* determine speed */
	

		/* check design characteristics
		   for a different number of rowers */

		printf ("     To try a different design or exit, enter 'x' \n");
		printf ("          (any other to change rowers for this design): ");
		}
	while ((newrow = getch()) != 'x' && newrow != 'X');
		/* tests to repeat sequence */
	printf ("\n\n");

	} /* end design() */

/***************          ***************/


findspeed()
	/* determine rowers needed for a given speed;
	   speeds for a given number of rowers;
	   report design summary */

	{
	float speedcalc(), speedchk();
		/* determine speed --
		   dash, chase, and cruise */

	char test;
		/* test for report requests */


	/* number of rowers, and horsepower available */

	printf ("\n\n");
	printf ("     Enter number of rowers: ");
	scanf ("%f", &rowers);

	printf ("\n");
	printf ("     Enter average number of rowers per bench \n");
	printf ("          ");
	printf ("(need not be an integer; e.g., \n");
	printf ("          ");
	printf ("a '3 + 2' quinquereme averages 2.5): ");

	scanf ("%f", &bench);
	benchfactor = pow (bench, 0.75) / bench;
		/* multiple rowers per bench less efficient */

	printf ("\n");
	printf ("Efficiency factor for ");
	printf ("%2.1f rowers per bench is ", bench);
	printf ("%2.0f percent \n\n", (benchfactor * 100));


	basehp = (rowers / ROWPWR) * benchfactor;
		/* horsepower available */

	printf ("     Enter desired speed: ");
	scanf ("%f", &speed);
	if (speed < 0.1)
		{
		speed = 6.5;
		printf ("(Error entry, speed set to 6.5 kts) \n");
		}
		/* default for error entry */


	/* horsepower for desired speed */

	hp = speedcalc();
	printf ("\n");

	printf ("     %2.1f horsepower required \n", hp);
	printf ("     wave ratio is: ");
	printf ("%3.2f \n\n\n", (wave / (wave + frict)));
		/* proportion of rowers' power going to
		   bow wave formation at desired speed */


	/* rowers needed for desired speed */

	printf ("     Rowers required for %2.1f knots: \n\n", speed);

	dashrow = (hp * ROWPWR) / benchfactor;
		/* rowers needed, dash stroke */
	printf ("     20-minute dash:   ");
	printf ("%1.0f rowers \n", dashrow);
	
	chaserow = dashrow / CHPWR;
		/* dividing by power fraction */
	printf ("     2-hour chase:     ");
	printf ("%1.0f rowers \n", chaserow);
	
	cruiserow = dashrow / CRPWR;
	printf ("     8-hour cruise:    ");
	printf ("%1.0f rowers \n", cruiserow);

	shiftrow = dashrow / SFPWR;
	printf ("     Rowing in shifts: ");
	printf ("%1.0f rowers \n\n\n", shiftrow);



	/* speed attainable with assigned rowers */

	printf ("     Maximum speeds for %1.0f rowers: \n\n", rowers);

	availhp = basehp;
	dash = speedchk();
	printf ("       20-minute dash   = %4.2f knots \n", dash);

	printf ("           (wave ratio is: ");
	printf ("%3.2f) \n", (wave / (wave + frict)));
		/* proportion of rowers' power going
		   to wave formation at actual top speed */

	availhp = basehp * CHPWR;
	chase = speedchk();
	printf ("       2-hour chase     = %4.2f knots \n", chase);
	
	availhp = basehp * CRPWR;
	cruise = speedchk();
	printf ("       8-hour cruise    = %4.2f knots \n", cruise);

	availhp = basehp * SFPWR;
	shift = speedchk();
	printf ("       Rowing in shifts = %4.2f knots \n\n", shift);



		/* computed maximum speed
		   for ship as designed */

	/* report ship summary if desired;
	   option to save to file */

	printf ("     To repeat specifications or ");
	printf ("save design to a file, type 'y': ");
	if ((test = getch()) == 'y' || test == 'Y')
		/* report summary to screen */
		report();
	else
		/* no report; continue */
		printf ("\n\n");
		

	} /* end findspeed() */

/***************          ***************/


float speedchk()
	/* determine speed for available rowers capacity;
	   simple step-up / step-down procedure */

	{
	float speedcalc();
		/* speed calculation formula */

	speed = 6.5;
		/* starting guess */

	hp = speedcalc();
		/* horsepower for initial test speed */

	if (hp > availhp)
		/* ship is slower; step down */
		while ((hp = speedcalc()) > availhp)
			speed = speed - 0.005;
	else
		/* ship is faster; step up */
		while ((hp = speedcalc()) < availhp)
			speed = speed + 0.005;

	return speed;

	} /* end speedchk() */	

/***************          ***************/


float speedcalc()
	/* speed calculation formula  
	
		(Based on W.L Rodgers, *Greek and Roman Naval Warfare,*
		page 31, notes 1 and 2 */

	{
	wave = cowave * pow (speed, 4.0);
		/* resistance due to wave formation */
	frict = cofriction * pow (speed, 1.83);
		/* wetted surface friction resistance */
	hp = ((wave + frict) * speed) / 331.5;
		/* horsepower required for speed */
	return hp;
		/* returns horsepower for given speed */

	} /* end speedcalc() */

/***************          ***************/


report()
	/* write report summary to screen */

	{
	char test;
		/* test for report requests */
	

	printf ("\n\n");
	printf ("%s \n\n     ", name);
	printf ("%2.0f tons ", displacement);
	printf ("normal displacement \n\n");

	printf ("Waterline dimensions: \n");
	printf ("     L: %5.0f feet\n", length);
	printf ("     B: %5.1f feet\n", beam);
	printf ("     D: %5.1f feet\n", depth);
	printf ("     (%3.2f block coefficient) \n\n", blockratio);

	printf ("     Maximum speeds for ");
	printf ("%1.0f rowers, ", rowers);
	printf ("average %2.1f per bench: \n\n", bench);

	printf ("           20-min dash      = %4.2f knots \n", dash);
	printf ("           2-hr chase       = %4.2f knots \n", chase);
	printf ("           8-hr cruise      = %4.2f knots \n", cruise);
	printf ("           Rowing in shifts = %4.2f knots \n\n", shift);


	/* save complete summary as file if desired */

	printf ("     ");
	printf ("To save specifications to file, type 'y': ");
	if ((test = getch()) == 'y' || test == 'Y')
		/* write full summary to file */
		savetype();
	printf ("\n\n");
	
	} /* end report() */

/***************          ***************/


savetype()
	/* write ship data to a file */

	{
	char chkfile;

	printf ("\n\n");
	printf ("Enter file name for ship or type: ");
	scanf ("%s", filename);

	shiptype = fopen (filename, "w");
		/* open file */

	fprintf (shiptype, "%s \n\n", name);
		/* ship type or name, and year laid down */


	/* general characteristics report */

	fprintf (shiptype, "Dimensions on waterline: \n");
	fprintf (shiptype, "     ");
	fprintf (shiptype, "Length, %2.0f ft x ", length);
	fprintf (shiptype, "Beam, %2.1f ft x ", beam);
	fprintf (shiptype, "Depth, %2.1f ft \n\n", depth);

	fprintf (shiptype, "     %2.1f ", displacement);
	fprintf (shiptype, "tons normal displacement ");
	fprintf (shiptype, "(submergence ");
	fprintf (shiptype, "%3.2f tons/inch) \n", submergence);

	fprintf (shiptype, "               wetted surface: ");
	fprintf (shiptype, "%1.0f ", (cofriction * 100.0));
	fprintf (shiptype, "square feet \n");

	fprintf (shiptype, "               block coefficient: ");
	fprintf (shiptype, "%0.2f \n", blockratio);
	fprintf (shiptype, "               sharpness coefficient: ");
	fprintf (shiptype, "%0.2f \n", cosharp);

	fprintf (shiptype, "               'M' = ");
	fprintf (shiptype, "%3.2f \n\n", (length / 3.28) /
					  pow (displacement, 0.333));

	fprintf (shiptype, "     %2.1f tons burden, ", burden);
	fprintf (shiptype, "carpenters' measure \n");

	fprintf (shiptype, "     %2.1f tons burden, ", custburden);
	fprintf (shiptype, "custom-house measure \n\n");


	/* performance report */

	fprintf (shiptype, "     Maximum speeds for ");
	fprintf (shiptype, "%1.0f rowers; ", rowers);
	fprintf (shiptype, "average %2.1f per bench: \n\n", bench);

	fprintf (shiptype, "          ");
	fprintf (shiptype, "20-min dash      = %4.2f knots \n", dash);

	fprintf (shiptype, "          ");
	fprintf (shiptype, " 2-hr chase      = %4.2f knots \n", chase);

	fprintf (shiptype, "          ");
	fprintf (shiptype, "8-hr cruise      = %4.2f knots \n", cruise);

	fprintf (shiptype, "          ");
	fprintf (shiptype, "Rowing in shifts = %4.2f knots \n\n", shift);


	fclose (shiptype);
		/* close file */

	} /* end savetype() */

/***************    END   ***************/

