/* Planet Mini-Simulator, v. 1.0 (6/9/99) */

	/* Copyright 1999 by Rick Robinson */

	#include <stdio.h>
	#include <conio.h>
	#include <bios.h>
	#include <stdlib.h>
	#include <float.h>
	#include <math.h>
	#include <ctype.h>


	#define TRUE            1
	#define FALSE           0


	#define PI              3.1416
	#define RADIAN         57.3

	#define ARC           180


	#define YEAR          360
	#define MONTH          30

	#define CELSIUS       273.0

	#define EARTHRAD      255.8
		/* radiative balance
		   temperature of Earth, K */

	#define GREENHOUSE     32.8
		/* average greenhouse
		   heating of Earth, K */

	#define STDPRESSURE  1013.0
		/* Earth surface atmospheric pressure,
		   in millibars */



float   luminosity, orbitdist, yearlength,
	eccentricity, apastron, periastron,
	diameter, area, pressure;

FILE	*fopen(), *planet;

char    world[20], planetname[50];



main()
	{
	char test;


	printf ("\n");
	printf ("Planet Mini-Simulator, v. 1.0 ");
	printf ("\n\n");
	printf ("     ");
	printf ("Copyright 1999 by Rick Robinson \n");
	printf ("     ");
	printf ("All rights reserved \n\n");
	printf ("     ");
	printf ("(e-mail: Lyonesse@compuserve.com) ");


	do {
		printf ("\n\n");

		cycle();

		printf ("To quit, enter 'X' ");
		printf ("(any other to continue): ");
		}
	while ((test = getch()) != 'x' && test != 'X');
	printf ("\n\n");
	printf ("\xad");
	printf ("Adios!\n\n");
	}

/*****          *****/

cycle()
	{
	float temperature();
	char checkchar;


	printf ("\n\n");

	flushall();

	printf ("Enter luminosity of parent star ");
	printf ("(Sun = 1.0): ");
	scanf ("%f", &luminosity);
	printf ("\n");

	printf ("Enter semi-major axis ");
	printf ("(average orbital distance) in AU: ");
	scanf ("%f", &orbitdist);
	printf ("\n");

	yearlength = sqrt (pow (orbitdist, 3.0));

	yearlength /= sqrt (pow (luminosity, 0.2));
		/* adjustment for stellar
		   mass-luminosity law */

	printf ("     ");
	printf ("Orbital period, %3.2f years \n\n", yearlength);


	printf ("Enter eccentricity of orbit ");
	printf ("(Earth = 0.017): ");
	scanf ("%f", &eccentricity);

	if (eccentricity >= 1.0)
		eccentricity *= 0.01;
		/* percentage input assumed */

	printf ("\n");

	if (eccentricity != 0.0)
		{
		apastron = orbitdist + (eccentricity * orbitdist);
		periastron = orbitdist - (eccentricity * orbitdist);

		printf ("     ");
		printf ("Apastron at %4.3f AU \n", apastron);

		printf ("     ");
		printf ("Periastron at %4.3f AU \n\n", periastron);

		printf ("     ");
		printf ("Approximate surface temperature: \n\n");

		printf ("          ");
		printf ("Periastron, %3.1f F \n", temperature (periastron));

		printf ("          ");
		printf ("Apastron, %3.1f F \n\n", temperature (apastron));

		printf ("          ");
		printf ("Average, %3.1f F \n\n", temperature (orbitdist));
		}
	else
		{
		printf ("     ");
		printf ("Approximate surface temperature: ");
		printf ("%3.1f F \n\n", temperature (orbitdist));
		}

	printf ("\n\n");

	printf ("Enter diameter of planet, in miles ");
	printf ("(Earth = 7950 mi): ");
	scanf ("%f", &diameter);
	printf ("\n");

	area = ((4.0 * PI) * pow ((diameter / 2.0), 2.0)) / 1000000.0;

	printf ("     ");
	printf ("Surface area, %3.1f million square miles \n\n", area);

	diameter /= 7950.0;


	printf ("     ");
	printf ("Estimated characteristics: \n\n");

	printf ("          ");
	printf ("Mass = ~ %4.2f x Earth \n", pow (diameter, 3.5));
	printf ("          ");
	printf ("Surface gravity = ~ %4.2f g \n",
					      pow (diameter, 1.5));
	printf ("          ");
	printf ("Density = ~ %4.2f gm / cm^3 \n\n",
				      pow (diameter, 0.5) * 5.52);

	pressure = pow (diameter, 3.0) * STDPRESSURE;

	printf ("     ");
	printf ("Typical atmospheric surface pressure: \n\n");
	printf ("          ");
	printf ("%1.0f millibars (Earth = 1013 mb) \n\n", pressure);

	printf ("     ");
	printf ("Typical ocean surface: \n\n");
	printf ("          ");
	printf ("%1.0f ", pow (0.71, pow ((1.0 / diameter), 1.75)) * 100.0);
	printf ("percent (Earth = 71 pct) \n\n");

	printf ("\n\n");
	printf ("To save as a file, enter 'Y' ");

	if ((checkchar = getch()) == 'y' || checkchar == 'Y')
		savefile();

	printf ("\n\n");
	}

/*****     *****/

characteristics()
	{
	fprintf (planet, "          ");
	fprintf (planet, "Diameter, %1.0f miles \n", (diameter * 7950.0));
	fprintf (planet, "          ");
	fprintf (planet, "Mass = ~ %4.2f x Earth \n", pow (diameter, 3.5));
	fprintf (planet, "          ");
	fprintf (planet, "Surface gravity = ~ %4.2f g \n",
					      pow (diameter, 1.5));
	fprintf (planet, "          ");
	fprintf (planet, "Density = ~ %4.2f gm / cm^3 \n\n",
				      pow (diameter, 0.5) * 5.52);

	/* surface pressure estimate is
	   (mass / surface) * surface gravity;

	   continentality estimate is approximated
	   from water proportional to mass,
	   average ocean depth square root
	   of relative water supply per area */

	}

/*****     *****/

ellipse()
	/* model an ellipse */

	{
	float   baserad, keplerrad, monthangle, theta, sunangle;

	int monthset;

	fprintf (planet, "      Time                  ");
	fprintf (planet, "Distance           Position \n\n");

	for (monthset = 1; monthset <= 12; monthset++)
		{
		monthangle = monthset * 30.0;

		theta = monthangle;

		baserad = (1.0 - pow (eccentricity, 2.0)) /
			  (1.0 + (eccentricity * cos (theta / RADIAN)));

		keplerrad = 2.0 - baserad;

		sunangle = asin ((baserad * sin (theta / RADIAN)) /
			   keplerrad);

		if ((cos (theta / RADIAN) * baserad) < -(2.0 * eccentricity))
			sunangle = PI - sunangle;

		if (sunangle < 0.0)
			sunangle = (2.0 * PI) + sunangle;

		sunangle *= RADIAN;


		fprintf (planet, " %9.2f years: ",
		((monthset * yearlength) / 12.0));

		fprintf (planet, "       ");
		fprintf (planet, "%8.3f AU ", (keplerrad *
						      orbitdist));

		fprintf (planet, "          ");
		fprintf (planet, "%3.0f deg \n", sunangle);
		}
	}

/*****     *****/

savefile()
	{
	float temperature();


	flushall();

	printf ("\n\n");
	printf ("Enter name of planet: ");
	gets (planetname);
	printf ("\n");

	printf ("Enter filename: ");
	scanf ("%s", world);
	planet = fopen (world, "w");
	printf ("\n\n");

	fprintf (planet, "\n");
	fprintf (planet, "          ");
	fprintf (planet, "%s", planetname);
	fprintf (planet, "\n\n\n");


	fprintf (planet, "     ");
	fprintf (planet, "Stellar luminosity, ");
	fprintf (planet, "%3.2f x Sun \n\n", luminosity);

	fprintf (planet, "     ");
	fprintf (planet, "Semi-major axis, ");
	fprintf (planet, "%3.2f AU \n", orbitdist);
	fprintf (planet, "     ");
	fprintf (planet, "Orbital period, %3.2f years \n", yearlength);

	fprintf (planet, "     ");
	fprintf (planet, "Orbital eccentricity, ");
	fprintf (planet, "%4.3f \n", eccentricity);

	if (eccentricity != 0.0)
		{
		fprintf (planet, "\n");

		fprintf (planet, "     ");
		fprintf (planet, "Apastron at %4.3f AU \n", apastron);

		fprintf (planet, "     ");
		fprintf (planet, "Periastron at ");
		fprintf (planet, "%4.3f AU \n\n", periastron);

		fprintf (planet, "          ");
		fprintf (planet, "Approximate surface temperature: \n\n");

		fprintf (planet, "               ");
		fprintf (planet, "Periastron, ");
		fprintf (planet, "%3.1f F \n", temperature (periastron));

		fprintf (planet, "               ");
		fprintf (planet, "Apastron, ");
		fprintf (planet, "%3.1f F \n\n", temperature (apastron));

		fprintf (planet, "               ");
		fprintf (planet, "Average, ");
		fprintf (planet, "%3.1f F \n\n\n", temperature (orbitdist));

		fprintf (planet, "     ");
		fprintf (planet, "Orbital characteristics: \n\n");

		ellipse();
		}
	else
		{
		fprintf (planet, "\n");
		fprintf (planet, "          ");
		fprintf (planet, "Approximate surface temperature: ");
		fprintf (planet, "%3.1f F \n\n", temperature (orbitdist));
		}

	fprintf (planet, "\n\n\n");

	fprintf (planet, "     ");
	fprintf (planet, "Diameter, %1.0f miles \n", (diameter * 7950.0));

	fprintf (planet, "     ");
	fprintf (planet, "Surface area, ");
	fprintf (planet, "%3.1f million square miles \n\n", area);

	fprintf (planet, "     ");
	fprintf (planet, "Estimated characteristics: \n\n");

	fprintf (planet, "          ");
	fprintf (planet, "Mass = ~ %4.2f x Earth \n", pow (diameter, 3.5));
	fprintf (planet, "          ");
	fprintf (planet, "Surface gravity = ~ %4.2f g \n",
					      pow (diameter, 1.5));
	fprintf (planet, "          ");
	fprintf (planet, "Density = ~ %4.2f gm / cm^3 \n\n",
				      pow (diameter, 0.5) * 5.52);

	fprintf (planet, "     ");
	fprintf (planet, "Typical atmospheric surface pressure: \n\n");
	fprintf (planet, "          ");
	fprintf (planet, "%1.0f millibars (Earth = 1013 mb) \n\n", pressure);

	fprintf (planet, "     ");
	fprintf (planet, "Typical ocean surface: \n\n");
	fprintf (planet, "          ");
	fprintf (planet, "%1.0f ", pow (0.71, pow ((1.0 / diameter), 1.75)) *
				   100.0);
	fprintf (planet, "percent (Earth = 71 pct) \n\n");

	fclose (planet);
	}

/*****     *****/

float temperature (distance)
	float distance;

	{
	float surftemp;


	surftemp = (1.0 / sqrt (distance)) *
		   pow (luminosity, 0.25);

	surftemp *= EARTHRAD;

	surftemp += GREENHOUSE;

	surftemp = ((surftemp - 273.0) * (9.0 / 5.0)) + 32.0;

	return (surftemp);
	}

/** END **/

