/* Planetary Climate Simulator v. 1.2 (6/9/99 - 4/15/01) */

	/* Copyright 1999, 2001 by Rick Robinson */

	#include <stdio.h>
	#include <conio.h>
	#include <bios.h>
	#include <stdlib.h>
	#include <float.h>
	#include <math.h>
	#include <ctype.h>


	#define TRUE            1
	#define FALSE           0


	#define PI              3.1416
	#define RADIAN         57.3

	#define ARC           180


	#define ECCINCR         0.01
	#define INCLINCR        0.5 /* ref 2.0, old 0.5 */
		/* monthly increments for adding
		   orbital eccentricity
		   and axial inclination */


	#define YEAR          360
	#define MONTH          30
	#define INTERVAL       30
	#define HOURS          24

	#define CELSIUS       273.0

	#define SOLCONST      395.6
		/* radiative balance temperature
		   for vertical sun at 1 AU */

	#define EARTHTEMP     288.5
		/* average temperature of Earth, K */

	#define STDPRESSURE  1013.0
		/* Earth surface atmospheric pressure,
		   in millibars */

	#define HEATRATE        1.12
		/* ocean surface heating (deg K)
		   from one day of vertical
		   solar radiation (value is
		   for top 25 meters of ocean;
		   this heating does not occur
		   on a daily basis, but
		   seasonally; same value is
		   applied to surface water cooling */

	#define HEATMOVE        0.015 /* ref 0.015 */
		/* maximum proportion of zonal
		   heat difference reduced by
		   daily meridional heat transport */

	#define EARTHLAND       0.29
		/* proportion of land surface on Earth */

	#define SNOWTEMP      273.0
		/* maximum persisting snow-cover
		   temperature: 0 C */

	#define ICETEMP       260.0 /* ref 260.0 */
		/* maximum sea ice
		   temperature: -13.0 C */

	#define ICEALBEDO       0.6 /* ref 0.6 */
		/* additional reflectivity
		   due to ice or snow surface
		   (albedo of ice or snow
		   itself is up to 0.9;
		   reduced value due to
		   atmospheric absorption, etc.) */




float   luminosity, starmass, orbitdist, eccentricity, eccset,
	precession, yearlength, diameter,
	axis, inclset, declination,
	distance [MONTH + 1], incline [MONTH + 1],
	verttemp, albedo, cloudiness, cloudalbedo, surfalbedo, totalbedo,
	insolation, area, landsurf, latitude, nextlat, zone [ARC],
	belt [ARC / 10], daysun [ARC], totalsun [ARC],
	reflect [ARC], intalbedo [ARC],
	temperature [ARC], greenheat [ARC],
	soltemp [ARC], avgtemp [ARC], avggrnht [ARC],

	heattrans, cloudiness,
	latentfactor, co2, co2heat, latentheat, pressure;

int     runyears, settletime, yearctr, degree, day, hour, ctr1, ctr2;

FILE	*fopen(), *planet;

char    world[20], planetname[50],
	initflag, globereptflag;


main()
	{
	char test;


	printf ("\n");
	printf ("Planetary Climate Simulator, v. 1.2 ");
	printf ("\n\n");
	printf ("     ");
	printf ("Copyright 1999, 2001 by Rick Robinson \n");
	printf ("     ");
	printf ("All rights reserved \n\n");
	printf ("     ");
	printf ("(e-mail: Lyonesse@compuserve.com) ");


	do {
		printf ("\n\n");

		cycle();

		printf ("To quit, type 'X' ");
		printf ("(any other to continue): ");
		}
	while ((test = getch()) != 'x' && test != 'X');
	printf ("\n\n");
	printf ("\xad");
	printf ("Adios!\n\n");
	}

/*****          *****/

cycle()
	{
	float setsun, settemp, weather();

	char checkchar;


	flushall();

	settletime = 0;

	printf ("\n\n");
	printf ("Enter name of planet: ");
	gets (planetname);
	printf ("\n");

	printf ("Enter filename: ");
	scanf ("%s", world);
	planet = fopen (world, "w");
	printf ("\n\n");

	fprintf (planet, "\n");
	fprintf (planet, "          ");
	fprintf (planet, "%s", planetname);
	fprintf (planet, "\n\n\n");


	printf ("Enter luminosity of parent star ");
	printf ("(Sun = 1.0): ");
	scanf ("%f", &luminosity);
	fprintf (planet, "     ");
	fprintf (planet, "Stellar luminosity, ");
	fprintf (planet, "%3.2f x Sun \n\n", luminosity);
	printf ("\n");


	printf ("To enter mass of star, type 'Y' \n");
	printf ("     ");
	printf ("(any other allows program \n");
	printf ("     ");
	printf ("to estimate star mass): ");

	if (((checkchar = getch()) == 'y') || (checkchar == 'Y'))
		{
		printf ("Y \n\n");

		printf ("Enter mass of parent star ");
		printf ("(Sun = 1.0): ");
		scanf ("%f", &starmass);
		}
	else
		{
		printf ("N \n\n");

		starmass = pow (luminosity, 0.2);
			/* stellar mass-luminosity relationship */
		}

	fprintf (planet, "     ");
	fprintf (planet, "Stellar mass, ");
	fprintf (planet, "%3.2f x Sun \n\n", starmass);
	printf ("\n");



	printf ("Enter semi-major axis ");
	printf ("(average orbital distance) in AU: ");
	scanf ("%f", &orbitdist);
	fprintf (planet, "     ");
	fprintf (planet, "Semi-major axis, ");
	fprintf (planet, "%3.2f AU \n", orbitdist);
	printf ("\n");

	yearlength = sqrt (pow (orbitdist, 3.0));
		/* Kepler's law */

	yearlength /= sqrt (starmass);
		/* a bit of Newton */

	printf ("     ");
	printf ("Orbital period, %3.2f years \n\n", yearlength);

	fprintf (planet, "     ");
	fprintf (planet, "Orbital period, %3.2f years \n", yearlength);

	printf ("Enter eccentricity of orbit ");
	printf ("(Earth = 0.017): ");
	scanf ("%f", &eccset);
	if (eccset >= 1.0)
		eccset *= 0.01;
		/* percentage input assumed */

	fprintf (planet, "     ");
	fprintf (planet, "Orbital eccentricity, ");
	fprintf (planet, "%4.3f \n\n", eccset);

	if (eccset != 0.0)
		{
		settletime = (eccset / (ECCINCR * 12.0)) + 6.5;

		printf ("\n");
		printf (" >> ");
		printf ("Note: %1i ", settletime);
		printf ("years needed to establish orbit \n\n\n");

		printf ("           ");
		printf ("Precession sets orientation of ellipse \n");
		printf ("     ");
		printf ("major axis.  Zero deg precesssion means start \n");
		printf ("     ");
		printf ("of year (by definition, ");
		printf ("northern winter solstice) \n");
		printf ("     ");
		printf ("is at apastron, greatest distance from star; \n");
		printf ("     ");
		printf ("180 deg means year starts at periastron. \n\n");

		printf ("Enter ellipse precession (Earth = 190 deg): ");
		scanf ("%f", &precession);
		fprintf (planet, "     ");
		fprintf (planet, "Orientation of orbital major axis, ");
		fprintf (planet, "%1.0f degrees \n\n", precession);
		printf ("\n");
		}
	else
		{
		precession = 0.0;

		fprintf (planet, "\n");
		printf ("\n\n");
		}


	printf ("Enter diameter of planet, in miles ");
	printf ("(Earth = 7950 mi): ");
	scanf ("%f", &diameter);
	diameter /= 7950.0;
	printf ("\n");

	printf ("     ");
	printf ("Surface gravity = ~ %4.2f g \n\n\n", pow (diameter, 1.5));

	characteristics();

	printf ("Enter inclination of planetary axis ");
	printf ("(Earth = 23.5 deg): ");
	scanf ("%f", &inclset);
	fprintf (planet, "     ");
	fprintf (planet, "Inclination of planetary axis, ");
	fprintf (planet, "%2.1f degrees \n\n", inclset);

	if ((fabs (inclset / (INCLINCR * 12.0)) + 0.5) > settletime)
		{
		settletime = fabs (inclset / (INCLINCR * 12.0)) + 0.5;

		printf ("\n");
		printf (" >> ");
		printf ("Note: %1i ", settletime);
		printf ("years needed to initialize inclination \n\n\n");

		}
	else
		printf ("\n\n");


	printf ("Enter percentage of ocean surface \n");
	printf ("    ");
	printf ("(Earth = 71 pct; ");
	printf ("typical value, %1.0f pct): ", pow ((1.0 - EARTHLAND), 
	                                       pow ((1.0 / diameter), 1.75)) * 
	                                       100.0);
					      
	scanf ("%f", &landsurf);

	fprintf (planet, "     ");
	fprintf (planet, "%1.0f percent ocean surface \n", landsurf);

	landsurf = 1.0 - (landsurf * 0.01);
		/* convert from ocean percent to land fraction */
		
	if (landsurf >= EARTHLAND)
		/* reduce differential effect
		   of land surface relative to Earth;
		   emperical rule */

		{
		landsurf = landsurf - EARTHLAND;
		landsurf *= 1.0 / (1.0 - EARTHLAND);
		landsurf = pow (landsurf, 1.5); /* ref 1.75 */
		landsurf *= 1.0 - EARTHLAND;
		landsurf += EARTHLAND;

		/*
		if (landsurf > 0.99)
			landsurf = 0.99;
		*/

		}
	else
		{
		landsurf *= 1.0 / EARTHLAND;
		landsurf = 1.0 - pow ((1.0 - landsurf), 1.75);
		landsurf *= EARTHLAND;
		}

	printf ("\n");


	printf ("Enter atmosphere surface pressure, in millibars \n");
	printf ("     ");
	printf ("Earth = 1013 mb; ");
	printf ("typical value, %1.0f mb): ", (pow (diameter, 3.0) *
					     STDPRESSURE));
	scanf ("%f", &pressure);

	fprintf (planet, "     ");
	fprintf (planet, "Atmospheric surface pressure, ");
	fprintf (planet, "%1.0f millibars \n", pressure);

	pressure /= STDPRESSURE;
	printf ("\n");

	printf ("Enter atmospheric CO2 level, in millibars ");
	printf ("(Earth = 0.37): ");
	scanf ("%f", &co2);
	fprintf (planet, "     ");
	fprintf (planet, "Atmospheric CO2, %3.2f millibars \n\n", co2);
	printf ("\n\n");

	co2green();

	printf ("Enter latent heat energy factor \n");
	printf ("     (effect of temperature on cloudiness; \n");
	printf ("     typical values, ~ 0.2 - 0.5): ");
	scanf ("%f", &latentfactor);
	fprintf (planet, "     ");
	fprintf (planet, "Latent heat energy factor \n");
	fprintf (planet, "          ");
	fprintf (planet, "(effect of temperature on cloudiness; \n");
	fprintf (planet, "          ");
	fprintf (planet, "typically 0.2 - 0.5); ");	 
	fprintf (planet, "%3.2f \n\n", latentfactor);
	printf ("\n");

	printf ("Enter relative meridional (north-south) ");
	printf ("heat transport (Earth = 1.0): ");
	scanf ("%f", &heattrans);

	heattrans *= HEATMOVE;

	fprintf (planet, "     ");
	fprintf (planet, "Meridional heat transport coefficient, ");
	fprintf (planet, "%5.4f ", heattrans);
	fprintf (planet, "(= %3.2f x Earth) ", (heattrans / HEATMOVE));
	fprintf (planet, "\n\n");
	printf ("\n\n");

	printf ("Enter run time to equilibrate ");
	printf ("(typically %1i-%1i years): ", (settletime + 5),
					       (settletime + 15));
	scanf ("%i", &runyears);
	printf ("\n");

	if (runyears < settletime)
		{
		printf (" >>>>> ");
		printf ("Run will not establish orbit ");
		printf ("or inclination parameters \n\n");
		}
	printf ("\n");

	printf ("To report full globe, type 'Y' \n");
	printf ("     ");
	printf ("(any other for more detailed \n");
	printf ("     ");
	printf ("report of one hemisphere): ");

	if (((checkchar = getch()) == 'y') || (checkchar == 'Y'))
		{
		globereptflag = TRUE;
		printf ("Y \n\n");
		}
	else
		{
		globereptflag = FALSE;
		printf ("N \n\n");

		printf ("To report southern hemisphere, type 'S' \n");
		printf ("     ");
		printf ("(any other for northern hemisphere): ");
		if (((checkchar = getch()) == 's') || (checkchar == 'S'))
			{
			inclset = -inclset;
			printf ("S \n\n");
			}
		else
			printf ("N \n\n");
		}

	initflag = TRUE;
	eccentricity = eccset;

	fprintf (planet, "\n\n");
	fprintf (planet, "Orbital characteristics:    ");

	fprintf (planet, "Apastron at %4.3f AU;    ", (orbitdist +
						     (eccentricity *
						     orbitdist)));

	fprintf (planet, "Periastron at %4.3f AU \n\n", (orbitdist -
						       (eccentricity *
						       orbitdist)));

	ellipse();
		/* report parameters */

	fprintf (planet, "\n\n\n");


	/* initialize */

	initflag = FALSE;

	eccentricity = 0.0;
	axis = 0.0;

	/*
	eccentricity = eccset;
	axis = inclset;
	*/

	ellipse();
		/* start with zero eccentricity and
		   axial inclination to minimize
		   "initialization shock" */



	fprintf (planet, "     ");
	fprintf (planet, "Surface area by latitude: \n");

	surface();

	printf ("\n\n");


	surfalbedo = 0.1 + (landsurf * 0.14);
		/* average ocean albedo, 0.1; average
		   land albedo, 0.24 */

	printf ("Surface albedo = %3.2f \n\n", surfalbedo);

	if ((orbitdist * (1.0 / sqrt (luminosity))) < 1.0)
		{
		cloudiness = weather (EARTHTEMP *
				     pow ((1.0 / (orbitdist *
				     (1.0 / sqrt (luminosity)))), 0.33));

		settemp = EARTHTEMP * pow ((1.0 / (orbitdist *
				      (1.0 / sqrt (luminosity)))), 0.33);

			/* limited cloudiness and temperature
			   increase for close-in planets
			   to minimize "initialization shock" */
		}
	else
		{
		cloudiness = weather (EARTHTEMP *
				     pow ((1.0 / (orbitdist *
				     (1.0 / sqrt (orbitdist)))), 0.5));

		settemp = EARTHTEMP;
		}

	printf ("     ");
	printf ("Initial albedo = %3.2f \n\n", albedo);

	setsun = 0.25 * pow ((1.0 / orbitdist), 0.25);

	for (ctr1 = 0; ctr1 < (ARC / 10); ctr1++)
		/* initialize to approximate values */
		{
		degree = (ctr1 * 10) + 5;

		daysun [degree] = setsun;
		temperature [degree] = settemp;
		}

	latentheat = 1.0;

	for (yearctr = 1; yearctr <= runyears; yearctr++)
		{
		printf ("     ");
		printf ("Year %1i: \n\n", yearctr);

		fprintf (planet, "          ");
		fprintf (planet, "==================================");

		/*
		fprintf (planet, "\n\n\n\n\n\n\n\n\n\n\n");
		fprintf (planet, "\n\n\n\n\n\n\n\n\n\n\n");
		*/

		fprintf (planet, "\n\n\n\n");

		/*
		if (yearctr == runyears)
			fprintf (planet, "\n\n\n\n\n");
		*/

		fprintf (planet, "                       ");
		fprintf (planet, "Year %1i -- \n\n", yearctr);

		season();

		fprintf (planet, "\n\n");
		}

	fclose (planet);
	}

/*****     *****/

characteristics()
	{
	fprintf (planet, "     ");
	fprintf (planet, "Diameter, %1.0f miles \n", (diameter * 7950.0));
	fprintf (planet, "     ");
	fprintf (planet, "Mass = ~ %4.2f x Earth \n", pow (diameter, 3.5));
	fprintf (planet, "     ");
	fprintf (planet, "Surface gravity = ~ %4.2f g \n",
					      pow (diameter, 1.5));
	fprintf (planet, "     ");
	fprintf (planet, "Density = ~ %4.2f gm / cm^3 \n\n",
				      pow (diameter, 0.5) * 5.52);

	/* surface pressure estimate is
	   (mass / surface) * surface gravity;

	   continentality estimate is approximated
	   from water proportional to mass,
	   average ocean depth square root
	   of relative water supply per area */

	}

/*****     *****/

surface()
	/* relative surface area of latitude belts --
	   finally replacing the old clunker */

	{
	float surfarea;
	int beltctr;


	area = 0.0;

	surfarea = ((4.0 * PI) *
		   pow (((diameter * 7950.0) / 2.0), 2.0)) /
		   1000000.0;

	for (beltctr = 0; beltctr < (ARC / 10); beltctr++)
		belt [beltctr] = 0.0;

	for (degree = 0; degree < ARC; degree += 10)
		{
		latitude = (degree - 90) / RADIAN;
		nextlat = (degree - 80) / RADIAN;

		belt [degree / 10] = (sin (nextlat) - sin (latitude)) / 2.0;
		}

	fprintf (planet, "\n");

	area = 0.0;

	fprintf (planet, "Latitude zone            Surface area");
	fprintf (planet, "            ");
	fprintf (planet, "Global cumulative \n\n");
	for (degree = 90; degree < 180; degree = degree + 10)
		{
		fprintf (planet, "  %2i-%2i deg ", (degree - 90),
						    (degree - 80));
		fprintf (planet, "......     ");
		fprintf (planet, "%4.1f mill. sq mi ", (belt [degree / 10] *
						 surfarea));

		area = area + (2.0 * belt [degree / 10]);
		fprintf (planet, "        ");
		fprintf (planet, "%2i deg ... %5.1f pct \n", (degree - 80),
							     (area * 100.0));
		}

	fprintf (planet, "\n");
	}

/*****     *****/

season()
	/* seasonal temperature change
	   incorporating insolation and
	   emperical heat retention model */

	{
	float  reptdist, radtemp, heatcapacity,
		tempchange, endchange,
		weather(),
		globesun, globetemp, globegrn,
		avgdaytemp, avgcloud,
		avgalbedo;


	avgcloud = 0.0;
	avgalbedo = 0.0;

	for (ctr1 = 0; ctr1 < 18; ctr1++)
		{
		degree = (ctr1 * 10) + 5;

		avgtemp [degree] = 0.0;
		totalsun [degree] = 0.0;
		avggrnht [degree] = 0.0;
		intalbedo [degree] = 0.0;
		}

	for (day = INTERVAL; day <= YEAR; day += INTERVAL)
		{
		declination = incline [day / MONTH];

		reptdist = distance [day / MONTH] * orbitdist;

		verttemp = (SOLCONST / sqrt (reptdist)) *
			   pow (luminosity, 0.25);


		avgdaytemp = 0.0;

		printf ("Month %2i \n", (day / MONTH));
		for (ctr2 = 0; ctr2 < 18; ctr2++)
			{
			printf (".");
			degree = (ctr2 * 10) + 5;

			daysun [degree] = 0.0;


			sunlight (reptdist);
				/* update daysun [degree] */


			totalsun [degree] += (daysun [degree] /
					     (YEAR / INTERVAL));



			heatcapacity = 1.0 - (landsurf * 0.93);
				/* energy needed to produce
				   a given heating; much greater
				   for oceans */

			reflect [degree] = albedo;

			if (temperature [degree] < SNOWTEMP)
				/* effects of snow/ice cover
				   on albedo and heat capacity
				   of surface */
				{
				if (temperature [degree] < ICETEMP)
					{
					heatcapacity /= 4.0 -
							(landsurf * 3.0);
						/* ref 3.0, 2.0 */

					if (ICEALBEDO > albedo)
						{
						reflect [degree] = ICEALBEDO;
						}
					}
				else
					{
					reflect [degree] = cloudalbedo +
							   ((1.0 -
							   (cloudiness /
							   100.0)) *
							   (((1.0 -
							   landsurf) *
							   0.1) +
							   (landsurf *
							   ICEALBEDO)));

					if (reflect [degree] < albedo)
						{
						reflect [degree] = albedo;
						}
					}
				}

			intalbedo [degree] += reflect [degree];

			greenheat [degree] = greenhouse
					     (temperature [degree]);

			avggrnht [degree] += greenheat [degree];

			tempchange = (((daysun [degree] *
				     (1.0 - reflect [degree])) -
				     pow (((temperature [degree] -
				     greenheat [degree]) / verttemp), 4.0)) *

				     ((HEATRATE / sqrt (yearlength)) *
				     (1.0 / heatcapacity))) *
				     (INTERVAL * yearlength);

			endchange = (((daysun [degree] *
				    (1.0 - reflect [degree])) -
				    pow ((((temperature [degree] +
				    tempchange) -
				    greenheat [degree]) / verttemp), 4.0)) *

				    ((HEATRATE / sqrt (yearlength)) *
				    (1.0 / heatcapacity))) *
				    (INTERVAL * yearlength);


			if (tempchange != 0)
				tempchange = (tempchange + endchange) / 2.0;

				/* difference between ingoing and
				   outgoing radiation, averaged
				   between initial and final
				   preliminary values, times a
				   constant (times interval) */


			radtemp = (pow (daysun [degree], 0.25) *
				  (verttemp *
				  pow ((1.0 - reflect [degree]), 0.25))) +
				  greenheat [degree];

			if (((temperature [degree] < radtemp) &&
			   ((temperature [degree] + tempchange) > radtemp)) ||
			   ((temperature [degree] > radtemp) &&
			   ((temperature [degree] + tempchange) < radtemp)))
				/* overshot or undershot */
				{
				/*
				printf ("\n Bump-up \n");
				fprintf (planet, "\n     Bump-up -- \n");
				*/

				temperature [degree] = radtemp;
				}
			else
				temperature [degree] += tempchange;

			}

		for (ctr2 = 0; ctr2 < 3; ctr2++)
			heattransfer (heattrans / 3.0);


		for (ctr2 = 0; ctr2 < 18; ctr2++)
			{
			degree = (ctr2 * 10) + 5;

			avgdaytemp += temperature [degree] * belt [ctr2];

			avgtemp [degree] += temperature [degree];
			}

		if ((yearctr == runyears) && ((day % MONTH) == 0))
			{
			fprintf (planet, "\n");
			fprintf (planet, "Month %2i:    ", (day / MONTH));

			fprintf (planet, "Distance = ");

			fprintf (planet, "%4.3f AU      ", reptdist);

			fprintf (planet, "Declination = ");
			fprintf (planet, "%4.1f deg ", fabs (declination));

			if (fabs (declination) < 0.05)
				fprintf (planet, "\n");
			else
				{
				if (((declination > 0.0) &&
				   (inclset > 0.0)) ||
				   ((declination < 0.0) &&
				   (inclset < 0.0)))
					{
					fprintf (planet, "N \n");
					}
				else
					{
					fprintf (planet, "S \n");
					}
				}

			fprintf (planet, "             ");
			fprintf (planet, "Avg temp = ");
			fprintf (planet, "%6.1f F      ", (((avgdaytemp -
							  CELSIUS) *
							  (9.0 / 5.0)) +
							  32.0));

			fprintf (planet, "Albedo = ");
			fprintf (planet, "%3.2f;     ", albedo);

			fprintf (planet, "Cloud cover = ");
			fprintf (planet, "%2.0f pct \n", cloudiness);

			printf ("  %2.0f percent cloud cover ", cloudiness);


			if (globereptflag)
				globereport (daysun, temperature);
			else
				hemisreport (daysun, temperature);

			fprintf (planet, "\n\n");
			if ((day % (MONTH * 2)) == 0)
				fprintf (planet, "\n");
			}

		printf ("\n");

		avgalbedo += albedo;
		avgcloud += cloudiness;

		cloudiness = weather (avgdaytemp);
			/* cloudiness (and albedo) are post-set
			   to affect the following month's
			   conditions */

		if ((((yearctr == 6) && (day >= precession)) ||
		   (yearctr > 6)) &&
		   (eccentricity < eccset))
			{
			if ((eccentricity += ECCINCR) >= eccset)
				{
				eccentricity = eccset;

				printf ("\n");
				printf (" >> ");
				printf ("Eccentricity established \n\n");
				}

			ellipse ();
			}

		if (fabs (axis) < fabs (inclset))
			{
			if (fabs (axis + INCLINCR) >= fabs (inclset))
				{
				axis = inclset;

				printf ("\n");
				printf (" >> ");
				printf ("inclination established \n\n");
				}
			else
				{
				if (inclset > 0.0)
					axis += INCLINCR;
				else
					axis -= INCLINCR;
				}
			ellipse ();
			}

		printf ("\n");
		}

	fprintf (planet, "\n");


	fprintf (planet, "          Year Summary -- \n\n");

	globegrn = totalbedo = globesun = globetemp = 0.0;

	avgalbedo /= (YEAR / INTERVAL);

	for (ctr1 = 0; ctr1 < 18; ctr1++)
		{
		degree = (ctr1 * 10) + 5;
		avgtemp [degree] /= (YEAR / INTERVAL);
		globetemp += avgtemp [degree] * belt [ctr1];
		globesun += totalsun [degree] * belt [ctr1];

		intalbedo [degree] /= (YEAR / INTERVAL);
		totalbedo += intalbedo [degree] * belt [ctr1];

		avggrnht [degree] /= (YEAR / INTERVAL);
		globegrn += avggrnht [degree] * belt [ctr1];
		}


	fprintf (planet, "Relative global insolation = ");

	fprintf (planet, "%4.2f \n", (4.0 * globesun));


	avgcloud /= (YEAR / INTERVAL);
	fprintf (planet, "Average cloud cover, ");
	fprintf (planet, "%1.0f percent; ", avgcloud);

	fprintf (planet, "integrated albedo, ");
	fprintf (planet, "%4.3f; ", totalbedo);

	fprintf (planet, "nonpolar, %4.3f \n\n", avgalbedo);

	printf ("Integrated albedo = %4.3f \n\n", totalbedo);
	printf ("Nonpolar albedo = %4.3f \n\n", avgalbedo);

	globetemp -= CELSIUS;

	fprintf (planet, "Average global temperature: ");
	fprintf (planet, "%3.2f C = ", globetemp);
	fprintf (planet, "%3.2f F \n", ((globetemp * (9.0 / 5.0)) + 32.0));

	printf ("Average global temperature, year %1i:  ", yearctr);
	printf ("%3.2f F \n\n\n", ((globetemp * (9.0 / 5.0)) + 32.0));

	fprintf (planet, "Integrated zonal greenhouse heating: ");
	fprintf (planet, "%3.2f K \n\n", globegrn);

	globetemp += CELSIUS;

	if (globereptflag)
		globereport (totalsun, avgtemp);
	else
		hemisreport (totalsun, avgtemp);



	fprintf (planet, "\n\n");

	fprintf (planet, "Grnh:");

	if (globereptflag)
		{
		fprintf (planet, "   ");
		for (ctr1 = 0; ctr1 < 19; ctr1 += 2)
			{
			if ((ctr1 == 4) || (ctr1 == 9) || (ctr1 == 14))
				++ctr1;
			if (ctr1 == 10)
				fprintf (planet, "    ");
			degree = (ctr1 * 10) + 5;
			fprintf (planet, " %6.1fK", avggrnht [degree]);
			}
		}
	else
		{
		for (ctr1 = 0; ctr1 < 9; ctr1++)
			{
			degree = (ctr1 * 10) + 95;
			fprintf (planet, " %6.1fK", avggrnht [degree]);
			}
		}
	}

/*****     *****/

sunlight (distance)
	float distance;

	/* much better model for
	   daily sunlight at a given latitude
	   and solar declination */

	{
	float   zenith, nadir,
		highsun, lowsun, midsun,
		radius, rotation,
		sunlevel, insolation;

	if (degree > (ARC / 2))
		zenith = ((ARC / 2.0) - (degree - 90)) + declination;
	else
		zenith = degree - declination;


	if (zenith > (ARC / 2.0))
		zenith = ARC - zenith;


	highsun = sin (zenith / RADIAN);

	if (degree > (ARC / 2))
		nadir = -(((ARC / 2.0) - (degree - 90)) - declination);
	else
		nadir = -(degree + declination);


	if (nadir < -(ARC / 2.0))
		{
		nadir += ARC;
		nadir = -nadir;
		}


	lowsun = sin (nadir / RADIAN);

	midsun = (highsun + lowsun) / 2.0;
	radius = (highsun - lowsun ) / 2.0;

	for (hour = 0; hour < HOURS; hour++)
		{
		rotation = ((hour - (HOURS / 4)) * (360 / HOURS)) / RADIAN;
		sunlevel = (sin (rotation) * radius) + midsun;

		if ((sunlevel > 0.0) && (zenith > 0.0))
			{
			daysun [degree] += (sunlevel *
					   (luminosity *
					   pow ((1.0 / distance), 2.0))) *
					   1.006; /* ref 1.006157; */

				/* constant is correction for
				   average integration error */
			}
		}

	daysun [degree] = daysun [degree] / HOURS;
	}

/*****     *****/

hemisreport (sun, temp)
	float sun[], temp[];

	{
	float repttemp;
	int ctr, lat;

	fprintf (planet, "\n");

	fprintf (planet, "Latitude");
	for (ctr = 0; ctr < 9; ctr++)
		{
		lat = (ctr * 10) + 95;
		fprintf (planet, " %2i", (lat - 90));
		if (inclset > 0.0)
			fprintf (planet, " N   ");
		else
			fprintf (planet, " S   ");
		}
	fprintf (planet, "\n\n");

	fprintf (planet, "Insol:  ");
	for (ctr = 0; ctr < 9; ctr++)
		{
		lat = (ctr * 10) + 95;
		fprintf (planet, " %4.2f   ", sun [lat]);
		}
	fprintf (planet, "\n");

	fprintf (planet, "Temp:");

	for (ctr = 0; ctr < 9; ctr++)
		{
		lat = (ctr * 10) + 95;

		repttemp = temp [lat] - CELSIUS;
		fprintf (planet, " %6.1fC", repttemp);
		}

	fprintf (planet, "\n");
	fprintf (planet, "     ");
	for (ctr = 0; ctr < 9; ctr++)
		{
		lat = (ctr * 10) + 95;
		repttemp = ((temp [lat] - CELSIUS) *
			   (9.0 / 5.0)) + 32.0;
		fprintf (planet, " %6.1fF", repttemp);
		}
	}

/*****     *****/

globereport (sun, temp)
	float sun[], temp[];

	{
	float repttemp;
	int ctr, lat;

	fprintf (planet, "\n");

	fprintf (planet, "Latitude  ");
	for (ctr = 0; ctr < 19; ctr += 2)
		{
		if ((ctr == 4) || (ctr == 9) || (ctr == 14))
			++ctr;
		if (ctr == 10)
			fprintf (planet, "    ");
		lat = (ctr * 10) + 5;

		fprintf (planet, "  %2i", abs (lat - 90));
		if ((lat - 90) > 0.0)
			fprintf (planet, " N  ");
		else
			fprintf (planet, " S  ");
		}
	fprintf (planet, "\n\n");

	fprintf (planet, "Insol:     ");
	for (ctr = 0; ctr < 19; ctr += 2)
		{
		if ((ctr == 4) || (ctr == 9) || (ctr == 14))
			++ctr;
		if (ctr == 10)
			fprintf (planet, "    ");
		lat = (ctr * 10) + 5;
		fprintf (planet, " %4.2f   ", sun [lat]);
		}
	fprintf (planet, "\n");

	fprintf (planet, "Temp:   ");

	for (ctr = 0; ctr < 19; ctr += 2)
		{
		if ((ctr == 4) || (ctr == 9) || (ctr == 14))
			++ctr;
		if (ctr == 10)
			fprintf (planet, "    ");
		lat = (ctr * 10) + 5;

		repttemp = temp [lat] - CELSIUS;
		fprintf (planet, " %6.1fC", repttemp);
		}

	fprintf (planet, "\n");
	fprintf (planet, "        ");
	for (ctr = 0; ctr < 19; ctr += 2)
		{
		if ((ctr == 4) || (ctr == 9) || (ctr == 14))
			++ctr;
		if (ctr == 10)
			fprintf (planet, "    ");
		lat = (ctr * 10) + 5;

		repttemp = ((temp [lat] - CELSIUS) *
			   (9.0 / 5.0)) + 32.0;
		fprintf (planet, " %6.1fF", repttemp);
		}
	}

/*****     *****/

heattransfer (transfer)
	/* meridional heat transfer, emperical model */

	float transfer;
	{
	float   tempdiff, heatshift,
		beltchange, nextchange,
		beltheatsink, nextheatsink,
		heatsinkratio, current,
		tempchange [ARC];

	int ctr, lat;


	for (ctr = 0; ctr < 18; ctr++)
		{
		lat = (ctr * 10) + 5;

		tempchange [lat] = 0.0;
		}

	for (ctr = 0; ctr < 17; ctr++)
		{
		/* main body computing temp diff */
		lat = (ctr * 10) + 5;

		beltheatsink = nextheatsink = 1.0;
		if (temperature [lat] < ICETEMP)
			{
			beltheatsink = 0.25 + (landsurf * 0.75);
			beltheatsink *= 1.0; /* ref 0.7 */
			}
		if (temperature [lat + 10] < ICETEMP)
			{
			nextheatsink = 0.25 + (landsurf * 0.75);
			nextheatsink *= 1.0;
			}
			/* reduced heat capacity of
			   ice-covered ocean surface */

		heatsinkratio = beltheatsink / nextheatsink;
			/* relative heat capacity
			   of latitude belts */

		tempdiff = temperature [lat] - temperature [lat + 10];

		heatshift = (transfer * (tempdiff *
			    ((INTERVAL * sqrt (yearlength)) / diameter))) *
			    (((temperature [lat] + temperature [lat + 10]) /
			    2.0) / EARTHTEMP);

			/* direct heat transfer based on constant
			   times average temperature */

		if ((temperature [lat] < ICETEMP) ||
		   (temperature [lat + 10] < ICETEMP))
			current = 0.0;
		else
			current = (1.0 - landsurf) * 0.7;
			/* added heat transfer effect
			   of ocean currents; no effect
			   if sea ice present */

		current *= sqrt (pressure);
			/* reduced wind movement of water;
			   higher wind velocity but
			   lower density yields
			   energy-momentum square root rule */

		current *= heatshift;

		heatshift = (((heatshift * 0.5) * sqrt (pressure)) +
			    ((heatshift * 0.5) * latentheat)) +
			    current;

			/* higher wind speed but lower density;
			   latent heat transfer unaffected */


		heatsinkratio *= belt [ctr] / belt [ctr + 1];
			/* factor in relative areas
			   of latitude belts */

		beltchange = -heatshift / sqrt (heatsinkratio);


		nextchange = heatshift * sqrt (heatsinkratio);
			/* temp change from heat transfer
			   governed by relative surface area
			   and heat capacity */

		/*
		fprintf (planet, "\n");
		fprintf (planet, "Temp diff %3.2f K \n", tempdiff);
		fprintf (planet, "Potential ");
		fprintf (planet, "heat shift %3.2f K \n", heatshift);
		fprintf (planet, "Heatsink ratio %3.2f \n\n", heatsinkratio);

		fprintf (planet, "%2i latitude change = ", lat);
		fprintf (planet, "%3.2f C \n", beltchange);
		fprintf (planet, "%2i latitude change = ", lat + 10);
		fprintf (planet, "%3.2f C \n", nextchange);
		*/

		if (fabs (beltchange) > (fabs (tempdiff) *
					(1.0 / (heatsinkratio + 1.0))))
			{
			beltchange = -tempdiff *
				     (1.0 / (heatsinkratio + 1.0));
			}
		if (fabs (nextchange) > (fabs (tempdiff) *
					(heatsinkratio /
					(heatsinkratio + 1.0))))
			{
			nextchange = tempdiff *
				     (heatsinkratio / (heatsinkratio + 1.0));
			}

		/*
		fprintf (planet, "     Adjusted -- \n");
		fprintf (planet, "%2i latitude change = ", lat);
		fprintf (planet, "%3.2f C \n", beltchange);
		fprintf (planet, "%2i latitude change = ", lat + 10);
		fprintf (planet, "%3.2f C \n", nextchange);
		*/

		tempchange [lat] += beltchange;

		tempchange [lat + 10] += nextchange;
		}


	for (ctr = 0; ctr < 18; ctr++)
		{
		lat = (ctr * 10) + 5;

		/*
		fprintf (planet, "%5i lat:  ", lat - 90);
		fprintf (planet, "temp change %6.2f K \n", tempchange [lat]);
		  */

		temperature [lat] += tempchange [lat];

		if (temperature [lat] < 1.0)
			temperature [lat] = 1.0;
		}

	}

/*****     *****/

co2green()
	/* emperical CO2 greenhouse approximation */
	{
	float greenfactor;

	greenfactor = pow (co2, 0.39);
	co2heat = greenfactor * 5.8;
	}

/*****     *****/

greenhouse (surftemp)
	float surftemp;

	/* emperical guesstimate of greenhouse heating */
	{
	float   logratio, reldensity, watervapor,
		greenfactor, greentemp, othergases;


	othergases = 1.25; /* ref 1.35 */

	if (surftemp < 1.0)
		surftemp = 1.0;

	logratio = log (surftemp / 288.0) /
		   (0.123 * pow ((surftemp / 288.0), 0.6));

		/* empirical "law" based on natural log
		   of temp ratio to produce change in
		   absolute humidity (water vapor partial
		   pressure) as temperature varies */


	reldensity = pow (10.0, logratio);


	watervapor = (reldensity * 16.0) * 0.68;

		/* saturation humidity 16 millibars
		   at standard temperature; 0.68 is
		   inferred average humidity
		   over oceans */

	watervapor *= (1.0 + landsurf) * (1.0 - landsurf);

		/* emperical formula based on relative
		   ocean and continental precipitation;
		   ocean humidity at standard value,
		   continental humidity declines in
		   proportion to land surface */

	/*
	printf ("Water vapor, %6.2f mb \n", watervapor);
	*/


	if (watervapor > (STDPRESSURE *
			 (500.0 * pow ((1.0 - landsurf), 1.5))))
		/* oceans fully evaporated,
		   no more water vapor available;
		   assumption that ocean depth
		   is proportional to square root
		   of surface area */
		{
		watervapor = STDPRESSURE *
			     (500.0 * pow ((1.0 - landsurf), 1.5));
		}


	greenfactor = pow (watervapor, 0.39);

	greentemp = (greenfactor * 11.0) + (co2heat + othergases);

	return (greentemp);
	}

/*****     *****/

float weather (surftemp)
	float surftemp;

	/* emperical guesstimate of greenhouse heating */
	{
	float   logratio, relvapor, cloudcover, reflect;


	if (surftemp < 1.0)
		surftemp = 1.0;

	logratio = log (surftemp / 288.0) /
		   (0.123 * pow ((surftemp / 288.0), 0.6));

	relvapor = pow (10.0, logratio);

		/* relvapor x 16 = saturation water vapor
		   in millibars at standard temperature
		   (note: inferred oceanic humidity is 0.68) */


	relvapor *= ((1.0 + landsurf) * (1.0 - landsurf)) / 0.92;

		/* emperical formula based on relative
		   ocean and continental precipitation;
		   0.92 is Earth's relative humidity,
		   compared to a thalassic planet */


	latentheat = pow (relvapor, latentfactor);

	cloudcover = latentheat * 0.47;
	if (cloudcover > 1.0)
		cloudcover = 1.0;

	reflect = (latentheat * 0.26) + 0.20;
	if (reflect > 0.8)
		reflect = 0.8;

	cloudalbedo = cloudcover * reflect;

	albedo = cloudalbedo +
		 ((1.0 - cloudcover) * surfalbedo);

	if (albedo < surfalbedo)
		albedo = surfalbedo;

	return (cloudcover * 100.0);
	}

/*****     *****/

ellipse()
	/* model an ellipse */

	{
	float   baserad, keplerrad, monthangle, theta, sunangle;

	int monthset;

	if (initflag)
		{
		fprintf (planet, "      Time                  ");
		fprintf (planet, "Distance           Position \n\n");
		}

	for (monthset = 1; monthset <= (YEAR / MONTH); monthset++)
		{
		monthangle = monthset * 30.0;

		theta = monthangle + precession;

		baserad = (1.0 - pow (eccentricity, 2.0)) /
			  (1.0 + (eccentricity * cos (theta / RADIAN)));

		keplerrad = 2.0 - baserad;

		sunangle = asin ((baserad * sin (theta / RADIAN)) /
			   keplerrad);

		if ((cos (theta / RADIAN) * baserad) < -(2.0 * eccentricity))
			sunangle = PI - sunangle;

		if (sunangle < 0.0)
			sunangle = (2.0 * PI) + sunangle;

		sunangle *= RADIAN;


		if (initflag)
			{
			fprintf (planet, " %9.2f years: ",
			((monthset * yearlength) / 12.0));

			fprintf (planet, "       ");
			fprintf (planet, "%8.3f AU ", (keplerrad *
						      orbitdist));

			fprintf (planet, "          ");
			fprintf (planet, "%3.0f deg \n", sunangle);
			}

		distance [monthset] = keplerrad;

		incline [monthset] = sin (((sunangle - precession) +
				     270.0) / RADIAN) * axis;
		}

	}

/** END **/

