/* Planetary Climate Simulator v. 1.3 (6/9/99 - 9/19/12) */

	/* Copyright 1999, 2001 by Rick Robinson */
	/* Linux and Win32 ports Copyright 2012 by Jon Brase */

	#include <stdio.h>
	#include <stdlib.h>
	#include <float.h>
	#include <math.h>
	#include <ctype.h>
	#ifdef _WIN32
		#include <curses.h>
	#else
		#include <ncurses.h>
	#endif

	#ifdef US_UNITS
	#define EARTH_DIAMETER 7950.0
	#define DIAM_UNIT "miles"
	#define DIAM_ABBREV "mi"
	#else
	#define EARTH_DIAMETER 12742.0
	#define DIAM_UNIT "kilometers"
	#define DIAM_ABBREV "km"
	#endif

	#define MAX_NAME_LENGTH 255
	#define MAX_FILENAME_LENGTH 255

	#define TRUE            1
	#define FALSE           0


	#define PI              3.1416
	#define RADIAN         57.3

	#define ARC           180


	#define ECCINCR         0.01
	#define INCLINCR        0.5 /* ref 2.0, old 0.5 */
		/* monthly increments for adding
		   orbital eccentricity
		   and axial inclination */


	#define YEAR          360
	#define MONTH          30
	#define INTERVAL       30
	#define HOURS          24

	#define CELSIUS       273.0

	#define SOLCONST      395.6
		/* radiative balance temperature
		   for vertical sun at 1 AU */

	#define EARTHTEMP     288.5
		/* average temperature of Earth, K */

	#define STDPRESSURE  1013.0
		/* Earth surface atmospheric pressure,
		   in millibars */

	#define HEATRATE        1.12
		/* ocean surface heating (deg K)
		   from one day of vertical
		   solar radiation (value is
		   for top 25 meters of ocean;
		   this heating does not occur
		   on a daily basis, but
		   seasonally; same value is
		   applied to surface water cooling */

	#define HEATMOVE        0.015 /* ref 0.015 */
		/* maximum proportion of zonal
		   heat difference reduced by
		   daily meridional heat transport */

	#define EARTHLAND       0.29
		/* proportion of land surface on Earth */

	#define SNOWTEMP      273.0
		/* maximum persisting snow-cover
		   temperature: 0 C */

	#define ICETEMP       260.0 /* ref 260.0 */
		/* maximum sea ice
		   temperature: -13.0 C */

	#define ICEALBEDO       0.6 /* ref 0.6 */
		/* additional reflectivity
		   due to ice or snow surface
		   (albedo of ice or snow
		   itself is up to 0.9;
		   reduced value due to
		   atmospheric absorption, etc.) */




double   luminosity, starmass, orbitdist, eccentricity, eccset,
	precession, yearlength, diameter,
	axis, inclset, declination,
	distance [MONTH + 1], incline [MONTH + 1],
	verttemp, landalbedo, albedo, cloudiness, cloudalbedo, surfalbedo, totalbedo,
	insolation, area, landsurf, latitude, nextlat, zone [ARC],
	belt [ARC / 10], daysun [ARC], totalsun [ARC],
	reflect [ARC], intalbedo [ARC],
	temperature [ARC], greenheat [ARC],
	soltemp [ARC], avgtemp [ARC], avggrnht [ARC],

	heattrans,
	latentfactor, co2, co2heat, latentheat, pressure;

int     runyears, settletime, yearctr, degree, day, hour, ctr1, ctr2;

FILE	*fopen(), *planet;

char    world[MAX_FILENAME_LENGTH+1], planetname[MAX_NAME_LENGTH+1],
	initflag, globereptflag;

void cycle(), characteristics(), co2green(), surface(), ellipse(), season(),
		sunlight(double distance), heattransfer(double transfer), globereport(double sun[], double temp[]), hemisreport(double sun[], double temp[]);

double weather(double surftemp), greenhouse(double surftemp);

main()
	{
	int test;

	initscr();
	scrollok(stdscr, TRUE);

	printw ("\n");
	printw ("Planetary Climate Simulator, v. 1.3 ");
	printw ("\n\n");
	printw ("     ");
	printw ("Copyright 1999, 2001 by Rick Robinson \n");
	printw ("     ");
	printw ("Linux and Win32 ports Copyright 2012 by Jon Brase \n");
	printw ("     ");
	printw ("All rights reserved \n\n");
	printw ("     ");
	printw ("(e-mail: Lyonesse@compuserve.com) ");
	refresh();


	do {
		nocbreak();
		echo();
		printw ("\n\n");
		refresh();

		cycle();

		printw ("To quit, type 'X' ");
		printw ("(any other to continue): ");
		cbreak();
		noecho();
		flushinp();
		refresh();
		}
	while ((test = getch()) != 'x' && test != 'X');
	printw ("\n\n");
	printw ("\xad");
	printw ("Adios!\n\n");
	refresh();
	endwin();
	}

/*****          *****/

void cycle()
	{
	double setsun, settemp;

	char checkchar;

	settletime = 0;

	printw ("\n\n");
	printw ("Enter name of planet: ");
	refresh();
	getnstr (planetname, MAX_NAME_LENGTH);
	printw ("\n");

	printw ("Enter filename: ");
	refresh();

#define LIMITED_STRING(s) MAKE_LSTRING_FORMAT(s)
#define MAKE_LSTRING_FORMAT(s) "%" #s "s"

	scanw (LIMITED_STRING(MAX_FILENAME_LENGTH), world);
	planet = fopen (world, "w");
	printw ("\n\n");

	fprintf (planet, "\n");
	fprintf (planet, "          ");
	fprintf (planet, "%s", planetname);
	fprintf (planet, "\n\n\n");

	printw ("Enter luminosity of parent star ");
	printw ("(Sun = 1.0): ");
	refresh();
	scanw ("%lf", &luminosity);
	fprintf (planet, "     ");
	fprintf (planet, "Stellar luminosity, ");
	fprintf (planet, "%3.2lf x Sun \n\n", luminosity);
	printw ("\n");

	printw ("To enter mass of star, type 'Y' \n");
	printw ("     ");
	printw ("(any other allows program \n");
	printw ("     ");
	printw ("to estimate star mass): ");
	refresh();

	cbreak();
	noecho();
	flushinp();

	if (((checkchar = getch()) == 'y') || (checkchar == 'Y'))
		{
		printw ("Y \n\n");

		printw ("Enter mass of parent star ");
		printw ("(Sun = 1.0): ");
		refresh();
		nocbreak();
		echo();
		scanw ("%lf", &starmass);
		}
	else
		{
		printw ("N \n\n");
		refresh();

		starmass = pow (luminosity, 0.2);
			/* stellar mass-luminosity relationship */
		}

	nocbreak();
	echo();

	fprintf (planet, "     ");
	fprintf (planet, "Stellar mass, ");
	fprintf (planet, "%3.2lf x Sun \n\n", starmass);
	printw ("\n");



	printw ("Enter semi-major axis ");
	printw ("(average orbital distance) in AU: ");
	refresh();
	scanw ("%lf", &orbitdist);
	fprintf (planet, "     ");
	fprintf (planet, "Semi-major axis, ");
	fprintf (planet, "%3.2lf AU \n", orbitdist);
	printw ("\n");

	yearlength = sqrt (pow (orbitdist, 3.0));
		/* Kepler's law */

	yearlength /= sqrt (starmass);
		/* a bit of Newton */

	printw ("     ");
	printw ("Orbital period, %3.2lf years \n\n", yearlength);

	fprintf (planet, "     ");
	fprintf (planet, "Orbital period, %3.2lf years \n", yearlength);

	printw ("Enter eccentricity of orbit ");
	printw ("(Earth = 0.017): ");
	refresh();
	scanw ("%lf", &eccset);
	if (eccset >= 1.0)
		eccset *= 0.01;
		/* percentage input assumed */

	fprintf (planet, "     ");
	fprintf (planet, "Orbital eccentricity, ");
	fprintf (planet, "%4.3lf \n\n", eccset);

	if (eccset != 0.0)
		{
		settletime = (eccset / (ECCINCR * 12.0)) + 6.5;

		printw ("\n");
		printw (" >> ");
		printw ("Note: %1i ", settletime);
		printw ("years needed to establish orbit \n\n\n");

		printw ("           ");
		printw ("Precession sets orientation of ellipse \n");
		printw ("     ");
		printw ("major axis.  Zero deg precesssion means start \n");
		printw ("     ");
		printw ("of year (by definition, ");
		printw ("northern winter solstice) \n");
		printw ("     ");
		printw ("is at apastron, greatest distance from star; \n");
		printw ("     ");
		printw ("180 deg means year starts at periastron. \n\n");

		printw ("Enter ellipse precession (Earth = 190 deg): ");
		refresh();
		scanw ("%lf", &precession);
		fprintf (planet, "     ");
		fprintf (planet, "Orientation of orbital major axis, ");
		fprintf (planet, "%1.0lf degrees \n\n", precession);
		printw ("\n");
		}
	else
		{
		precession = 0.0;

		fprintf (planet, "\n");
		printw ("\n\n");
		}


	printw ("Enter diameter of planet, in " DIAM_UNIT " ");
	printw ("(Earth = %.0lf " DIAM_ABBREV "): ", EARTH_DIAMETER);
	refresh();
	scanw ("%lf", &diameter);
	diameter /= EARTH_DIAMETER;
	printw ("\n");

	printw ("     ");
	printw ("Surface gravity = ~ %4.2lf g \n\n\n", pow (diameter, 1.5));

	characteristics();

	printw ("Enter inclination of planetary axis ");
	printw ("(Earth = 23.5 deg): ");
	refresh();
	scanw ("%lf", &inclset);
	fprintf (planet, "     ");
	fprintf (planet, "Inclination of planetary axis, ");
	fprintf (planet, "%2.1lf degrees \n\n", inclset);

	if ((fabs (inclset / (INCLINCR * 12.0)) + 0.5) > settletime)
		{
		settletime = fabs (inclset / (INCLINCR * 12.0)) + 0.5;

		printw ("\n");
		printw (" >> ");
		printw ("Note: %1i ", settletime);
		printw ("years needed to initialize inclination \n\n\n");
		refresh();
		}
	else
		printw ("\n\n");


	printw ("Enter average land albedo \n");
	printw ("    ");
	printw ("(Typical value, 0.24 percent): ");

	refresh();

	scanw("%lf", &landalbedo);

	printw ("Enter percentage of ocean surface \n");
	printw ("    ");
	printw ("(Earth = 71 pct; ");
	printw ("typical value, %1.0lf pct): ", pow ((1.0 - EARTHLAND),
	                                       pow ((1.0 / diameter), 1.75)) *
	                                       100.0);
	refresh();

	scanw ("%lf", &landsurf);

	fprintf (planet, "     ");
	fprintf (planet, "%1.0lf percent ocean surface \n", landsurf);

	landsurf = 1.0 - (landsurf * 0.01);
		/* convert from ocean percent to land fraction */

	if (landsurf >= EARTHLAND)
		/* reduce differential effect
		   of land surface relative to Earth;
		   emperical rule */

		{
		landsurf = landsurf - EARTHLAND;
		landsurf *= 1.0 / (1.0 - EARTHLAND);
		landsurf = pow (landsurf, 1.5); /* ref 1.75 */
		landsurf *= 1.0 - EARTHLAND;
		landsurf += EARTHLAND;

		/*
		if (landsurf > 0.99)
			landsurf = 0.99;
		*/

		}
	else
		{
		landsurf *= 1.0 / EARTHLAND;
		landsurf = 1.0 - pow ((1.0 - landsurf), 1.75);
		landsurf *= EARTHLAND;
		}

	printw ("\n");


	printw ("Enter atmosphere surface pressure, in millibars \n");
	printw ("     ");
	printw ("Earth = 1013 mb; ");
	printw ("typical value, %1.0lf mb): ", (pow (diameter, 3.0) *
					     STDPRESSURE));
	refresh();
	scanw ("%lf", &pressure);

	fprintf (planet, "     ");
	fprintf (planet, "Atmospheric surface pressure, ");
	fprintf (planet, "%1.0lf millibars \n", pressure);

	pressure /= STDPRESSURE;
	printw ("\n");

	printw ("Enter atmospheric CO2 level, in millibars ");
	printw ("(Earth = 0.37): ");
	refresh();
	scanw ("%lf", &co2);
	fprintf (planet, "     ");
	fprintf (planet, "Atmospheric CO2, %3.2lf millibars \n\n", co2);
	printw ("\n\n");

/*void*/ co2green();

	printw ("Enter latent heat energy factor \n");
	printw ("     (effect of temperature on cloudiness; \n");
	printw ("     typical values, ~ 0.2 - 0.5): ");
	refresh();

	scanw ("%lf", &latentfactor);
	fprintf (planet, "     ");
	fprintf (planet, "Latent heat energy factor \n");
	fprintf (planet, "          ");
	fprintf (planet, "(effect of temperature on cloudiness; \n");
	fprintf (planet, "          ");
	fprintf (planet, "typically 0.2 - 0.5); ");
	fprintf (planet, "%3.2lf \n\n", latentfactor);
	printw ("\n");

	printw ("Enter relative meridional (north-south) ");
	printw ("heat transport (Earth = 1.0): ");
	refresh();
	scanw ("%lf", &heattrans);

	heattrans *= HEATMOVE;

	fprintf (planet, "     ");
	fprintf (planet, "Meridional heat transport coefficient, ");
	fprintf (planet, "%5.4lf ", heattrans);
	fprintf (planet, "(= %3.2lf x Earth) ", (heattrans / HEATMOVE));
	fprintf (planet, "\n\n");
	printw ("\n\n");

	printw ("Enter run time to equilibrate ");
	printw ("(typically %1i-%1i years): ", (settletime + 5),
					       (settletime + 15));
	refresh();
	scanw ("%i", &runyears);
	printw ("\n");

	if (runyears < settletime)
		{
		printw (" >>>>> ");
		printw ("Run will not establish orbit ");
		printw ("or inclination parameters \n\n");
		}
	printw ("\n");

	printw ("To report full globe, type 'Y' \n");
	printw ("     ");
	printw ("(any other for more detailed \n");
	printw ("     ");
	printw ("report of one hemisphere): ");
	refresh();

	cbreak();
	noecho();
	flushinp();

	if (((checkchar = getch()) == 'y') || (checkchar == 'Y'))
		{
		globereptflag = TRUE;
		printw ("Y \n\n");
		}
	else
		{
		globereptflag = FALSE;
		printw ("N \n\n");

		printw ("To report southern hemisphere, type 'S' \n");
		printw ("     ");
		printw ("(any other for northern hemisphere): ");
		refresh();
		if (((checkchar = getch()) == 's') || (checkchar == 'S'))
			{
			inclset = -inclset;
			printw ("S \n\n");
			}
		else
			printw ("N \n\n");
		}
	refresh();
	nocbreak();
	echo();

	initflag = TRUE;
	eccentricity = eccset;

	fprintf (planet, "\n\n");
	fprintf (planet, "Orbital characteristics:    ");

	fprintf (planet, "Apastron at %4.3lf AU;    ", (orbitdist +
						     (eccentricity *
						     orbitdist)));

	fprintf (planet, "Periastron at %4.3lf AU \n\n", (orbitdist -
						       (eccentricity *
						       orbitdist)));

	ellipse();
		/* report parameters */

	fprintf (planet, "\n\n\n");


	/* initialize */

	initflag = FALSE;

	eccentricity = 0.0;
	axis = 0.0;

	/*
	eccentricity = eccset;
	axis = inclset;
	*/

	ellipse();
		/* start with zero eccentricity and
		   axial inclination to minimize
		   "initialization shock" */



	fprintf (planet, "     ");
	fprintf (planet, "Surface area by latitude: \n");

	surface();

	printw ("\n\n");


	surfalbedo = 0.1 + (landsurf * (landalbedo - 0.1));
		/* average ocean albedo, 0.1; average
		   land albedo, 0.24 */

	printw ("Surface albedo = %3.2lf \n\n", surfalbedo);
	fprintf(planet, "Surface albedo = %3.2lf \n\n", surfalbedo);

	if ((orbitdist * (1.0 / sqrt (luminosity))) < 1.0)
		{
		cloudiness = weather (EARTHTEMP *
				     pow ((1.0 / (orbitdist *
				     (1.0 / sqrt (luminosity)))), 0.33));

		settemp = EARTHTEMP * pow ((1.0 / (orbitdist *
				      (1.0 / sqrt (luminosity)))), 0.33);

			/* limited cloudiness and temperature
			   increase for close-in planets
			   to minimize "initialization shock" */
		}
	else
		{
		cloudiness = weather (EARTHTEMP *
				     pow ((1.0 / (orbitdist *
				     (1.0 / sqrt (orbitdist)))), 0.5));

		settemp = EARTHTEMP;
		}

	printw ("     ");
	printw ("Initial albedo = %3.2lf \n\n", albedo);

	setsun = 0.25 * pow ((1.0 / orbitdist), 0.25);

	for (ctr1 = 0; ctr1 < (ARC / 10); ctr1++)
		/* initialize to approximate values */
		{
		degree = (ctr1 * 10) + 5;

		daysun [degree] = setsun;
		temperature [degree] = settemp;
		}

	latentheat = 1.0;

	refresh();

	for (yearctr = 1; yearctr <= runyears; yearctr++)
		{
		printw ("     ");
		printw ("Year %1i: \n\n", yearctr);

		fprintf (planet, "          ");
		fprintf (planet, "==================================");

		/*
		fprintf (planet, "\n\n\n\n\n\n\n\n\n\n\n");
		fprintf (planet, "\n\n\n\n\n\n\n\n\n\n\n");
		*/

		fprintf (planet, "\n\n\n\n");

		/*
		if (yearctr == runyears)
			fprintf (planet, "\n\n\n\n\n");
		*/

		fprintf (planet, "                       ");
		fprintf (planet, "Year %1i -- \n\n", yearctr);

		season();

		fprintf (planet, "\n\n");
		refresh();
		}

	fclose (planet);
	}

/*****     *****/

void characteristics()
	{
	fprintf (planet, "     ");
	fprintf (planet, "Diameter, %1.0lf " DIAM_UNIT " \n", (diameter * EARTH_DIAMETER));
	fprintf (planet, "     ");
	fprintf (planet, "Mass = ~ %4.2lf x Earth \n", pow (diameter, 3.5));
	fprintf (planet, "     ");
	fprintf (planet, "Surface gravity = ~ %4.2lf g \n",
					      pow (diameter, 1.5));
	fprintf (planet, "     ");
	fprintf (planet, "Density = ~ %4.2lf gm / cm^3 \n\n",
				      pow (diameter, 0.5) * 5.52);

	/* surface pressure estimate is
	   (mass / surface) * surface gravity;

	   continentality estimate is approximated
	   from water proportional to mass,
	   average ocean depth square root
	   of relative water supply per area */

	}

/*****     *****/

void surface()
	/* relative surface area of latitude belts --
	   finally replacing the old clunker */

	{
	double surfarea;
	int beltctr;


	area = 0.0;

	surfarea = ((4.0 * PI) *
		   pow (((diameter * EARTH_DIAMETER) / 2.0), 2.0)) /
		   1000000.0;

	for (beltctr = 0; beltctr < (ARC / 10); beltctr++)
		belt [beltctr] = 0.0;

	for (degree = 0; degree < ARC; degree += 10)
		{
		latitude = (degree - 90) / RADIAN;
		nextlat = (degree - 80) / RADIAN;

		belt [degree / 10] = (sin (nextlat) - sin (latitude)) / 2.0;
		}

	fprintf (planet, "\n");

	area = 0.0;

	fprintf (planet, "Latitude zone            Surface area");
	fprintf (planet, "            ");
	fprintf (planet, "Global cumulative \n\n");
	for (degree = 90; degree < 180; degree = degree + 10)
		{
		fprintf (planet, "  %2i-%2i deg ", (degree - 90),
						    (degree - 80));
		fprintf (planet, "......     ");
		fprintf (planet, "%4.1lf mill. sq " DIAM_ABBREV " ", (belt [degree / 10] *
						 surfarea));

		area = area + (2.0 * belt [degree / 10]);
		fprintf (planet, "        ");
		fprintf (planet, "%2i deg ... %5.1lf pct \n", (degree - 80),
							     (area * 100.0));
		}

	fprintf (planet, "\n");
	}

/*****     *****/

void season()
	/* seasonal temperature change
	   incorporating insolation and
	   emperical heat retention model */

	{
	double  reptdist, radtemp, heatcapacity,
		tempchange, endchange,
		globesun, globetemp, globegrn,
		avgdaytemp, avgcloud,
		avgalbedo;


	avgcloud = 0.0;
	avgalbedo = 0.0;

	for (ctr1 = 0; ctr1 < 18; ctr1++)
		{
		degree = (ctr1 * 10) + 5;

		avgtemp [degree] = 0.0;
		totalsun [degree] = 0.0;
		avggrnht [degree] = 0.0;
		intalbedo [degree] = 0.0;
		}

	for (day = INTERVAL; day <= YEAR; day += INTERVAL)
		{
		declination = incline [day / MONTH];

		reptdist = distance [day / MONTH] * orbitdist;

		verttemp = (SOLCONST / sqrt (reptdist)) *
			   pow (luminosity, 0.25);


		avgdaytemp = 0.0;

		printw ("Month %2i \n", (day / MONTH));
		for (ctr2 = 0; ctr2 < 18; ctr2++)
			{
			printw (".");
			degree = (ctr2 * 10) + 5;

			daysun [degree] = 0.0;


			sunlight (reptdist);
				/* update daysun [degree] */


			totalsun [degree] += (daysun [degree] /
					     (YEAR / INTERVAL));



			heatcapacity = 1.0 - (landsurf * 0.93);
				/* energy needed to produce
				   a given heating; much greater
				   for oceans */

			reflect [degree] = albedo;

			if (temperature [degree] < SNOWTEMP)
				/* effects of snow/ice cover
				   on albedo and heat capacity
				   of surface */
				{
				if (temperature [degree] < ICETEMP)
					{
					heatcapacity /= 4.0 -
							(landsurf * 3.0);
						/* ref 3.0, 2.0 */

					if (ICEALBEDO > albedo)
						{
						reflect [degree] = ICEALBEDO;
						}
					}
				else
					{
					reflect [degree] = cloudalbedo +
							   ((1.0 -
							   (cloudiness /
							   100.0)) *
							   (((1.0 -
							   landsurf) *
							   0.1) +
							   (landsurf *
							   ICEALBEDO)));

					if (reflect [degree] < albedo)
						{
						reflect [degree] = albedo;
						}
					}
				}

			intalbedo [degree] += reflect [degree];

			greenheat [degree] = greenhouse
					     (temperature [degree]);

			avggrnht [degree] += greenheat [degree];

			tempchange = (((daysun [degree] *
				     (1.0 - reflect [degree])) -
				     pow (((temperature [degree] -
				     greenheat [degree]) / verttemp), 4.0)) *

				     ((HEATRATE / sqrt (yearlength)) *
				     (1.0 / heatcapacity))) *
				     (INTERVAL * yearlength);

			endchange = (((daysun [degree] *
				    (1.0 - reflect [degree])) -
				    pow ((((temperature [degree] +
				    tempchange) -
				    greenheat [degree]) / verttemp), 4.0)) *

				    ((HEATRATE / sqrt (yearlength)) *
				    (1.0 / heatcapacity))) *
				    (INTERVAL * yearlength);


			if (tempchange != 0)
				tempchange = (tempchange + endchange) / 2.0;

				/* difference between ingoing and
				   outgoing radiation, averaged
				   between initial and final
				   preliminary values, times a
				   constant (times interval) */


			radtemp = (pow (daysun [degree], 0.25) *
				  (verttemp *
				  pow ((1.0 - reflect [degree]), 0.25))) +
				  greenheat [degree];

			if (((temperature [degree] < radtemp) &&
			   ((temperature [degree] + tempchange) > radtemp)) ||
			   ((temperature [degree] > radtemp) &&
			   ((temperature [degree] + tempchange) < radtemp)))
				/* overshot or undershot */
				{
				/*
				printw ("\n Bump-up \n");
				refresh();
				fprintf (planet, "\n     Bump-up -- \n");
				*/

				temperature [degree] = radtemp;
				}
			else
				temperature [degree] += tempchange;

			}

		for (ctr2 = 0; ctr2 < 3; ctr2++)
			heattransfer (heattrans / 3.0);


		for (ctr2 = 0; ctr2 < 18; ctr2++)
			{
			degree = (ctr2 * 10) + 5;

			avgdaytemp += temperature [degree] * belt [ctr2];

			avgtemp [degree] += temperature [degree];
			}

		if ((yearctr == runyears) && ((day % MONTH) == 0))
			{
			fprintf (planet, "\n");
			fprintf (planet, "Month %2i:    ", (day / MONTH));

			fprintf (planet, "Distance = ");

			fprintf (planet, "%4.3lf AU      ", reptdist);

			fprintf (planet, "Declination = ");
			fprintf (planet, "%4.1lf deg ", fabs (declination));

			if (fabs (declination) < 0.05)
				fprintf (planet, "\n");
			else
				{
				if (((declination > 0.0) &&
				   (inclset > 0.0)) ||
				   ((declination < 0.0) &&
				   (inclset < 0.0)))
					{
					fprintf (planet, "N \n");
					}
				else
					{
					fprintf (planet, "S \n");
					}
				}

			fprintf (planet, "             ");
			fprintf (planet, "Avg temp = ");
#ifdef US_UNITS
			fprintf (planet, "%6.1lf F      ", (((avgdaytemp -
							  CELSIUS) *
							  (9.0 / 5.0)) +
							  32.0));
#else
			fprintf (planet, "%6.1lf C     ", (avgdaytemp - CELSIUS));
#endif

			fprintf (planet, "Albedo = ");
			fprintf (planet, "%3.2lf;     ", albedo);

			fprintf (planet, "Cloud cover = ");
			fprintf (planet, "%2.0lf pct \n", cloudiness);

			printw ("  %2.0lf percent cloud cover ", cloudiness);


			if (globereptflag)
				globereport (daysun, temperature);
			else
				hemisreport (daysun, temperature);

			fprintf (planet, "\n\n");
			if ((day % (MONTH * 2)) == 0)
				fprintf (planet, "\n");
			}

		printw ("\n");

		avgalbedo += albedo;
		avgcloud += cloudiness;

		cloudiness = weather (avgdaytemp);
			/* cloudiness (and albedo) are post-set
			   to affect the following month's
			   conditions */

		if ((((yearctr == 6) && (day >= precession)) ||
		   (yearctr > 6)) &&
		   (eccentricity < eccset))
			{
			if ((eccentricity += ECCINCR) >= eccset)
				{
				eccentricity = eccset;

				printw ("\n");
				printw (" >> ");
				printw ("Eccentricity established \n\n");
				refresh();
				}

			ellipse ();
			}

		if (fabs (axis) < fabs (inclset))
			{
			if (fabs (axis + INCLINCR) >= fabs (inclset))
				{
				axis = inclset;

				printw ("\n");
				printw (" >> ");
				printw ("inclination established \n\n");
				refresh();
				}
			else
				{
				if (inclset > 0.0)
					axis += INCLINCR;
				else
					axis -= INCLINCR;
				}
			ellipse ();
			}

		printw ("\n");
		refresh();
		}

	fprintf (planet, "\n");


	fprintf (planet, "          Year Summary -- \n\n");

	globegrn = totalbedo = globesun = globetemp = 0.0;

	avgalbedo /= (YEAR / INTERVAL);

	for (ctr1 = 0; ctr1 < 18; ctr1++)
		{
		degree = (ctr1 * 10) + 5;
		avgtemp [degree] /= (YEAR / INTERVAL);
		globetemp += avgtemp [degree] * belt [ctr1];
		globesun += totalsun [degree] * belt [ctr1];

		intalbedo [degree] /= (YEAR / INTERVAL);
		totalbedo += intalbedo [degree] * belt [ctr1];

		avggrnht [degree] /= (YEAR / INTERVAL);
		globegrn += avggrnht [degree] * belt [ctr1];
		}


	fprintf (planet, "Relative global insolation = ");

	fprintf (planet, "%4.2lf \n", (4.0 * globesun));


	avgcloud /= (YEAR / INTERVAL);
	fprintf (planet, "Average cloud cover, ");
	fprintf (planet, "%1.0lf percent; ", avgcloud);

	fprintf (planet, "integrated albedo, ");
	fprintf (planet, "%4.3lf; ", totalbedo);

	fprintf (planet, "nonpolar, %4.3lf \n\n", avgalbedo);

	printw ("Integrated albedo = %4.3lf \n\n", totalbedo);
	printw ("Nonpolar albedo = %4.3lf \n\n", avgalbedo);

	globetemp -= CELSIUS;

	fprintf (planet, "Average global temperature: ");
	fprintf (planet, "%3.2lf C = ", globetemp);
	fprintf (planet, "%3.2lf F \n", ((globetemp * (9.0 / 5.0)) + 32.0));

	printw ("Average global temperature, year %1i:  ", yearctr);
#ifdef US_UNITS
	printw ("%3.2lf F \n\n\n", ((globetemp * (9.0 / 5.0)) + 32.0));
#else
	printw ("%3.2lf C \n\n\n", globetemp);
#endif
	refresh();

	fprintf (planet, "Integrated zonal greenhouse heating: ");
	fprintf (planet, "%3.2lf K \n\n", globegrn);

	globetemp += CELSIUS;

	if (globereptflag)
		globereport (totalsun, avgtemp);

	else
		hemisreport (totalsun, avgtemp);



	fprintf (planet, "\n\n");

	fprintf (planet, "Grnh:");

	if (globereptflag)
		{
		fprintf (planet, "   ");
		for (ctr1 = 0; ctr1 < 19; ctr1 += 2)
			{
			if ((ctr1 == 4) || (ctr1 == 9) || (ctr1 == 14))
				++ctr1;
			if (ctr1 == 10)
				fprintf (planet, "    ");
			degree = (ctr1 * 10) + 5;
			fprintf (planet, " %6.1lfK", avggrnht [degree]);
			}
		}
	else
		{
		for (ctr1 = 0; ctr1 < 9; ctr1++)
			{
			degree = (ctr1 * 10) + 95;
			fprintf (planet, " %6.1lfK", avggrnht [degree]);
			}
		}
	}

/*****     *****/

void sunlight (double distance)

	/* much better model for
	   daily sunlight at a given latitude
	   and solar declination */

	{
	double   zenith, nadir,
		highsun, lowsun, midsun,
		radius, rotation,
		sunlevel, insolation;

	if (degree > (ARC / 2))
		zenith = ((ARC / 2.0) - (degree - 90)) + declination;
	else
		zenith = degree - declination;


	if (zenith > (ARC / 2.0))
		zenith = ARC - zenith;


	highsun = sin (zenith / RADIAN);

	if (degree > (ARC / 2))
		nadir = -(((ARC / 2.0) - (degree - 90)) - declination);
	else
		nadir = -(degree + declination);


	if (nadir < -(ARC / 2.0))
		{
		nadir += ARC;
		nadir = -nadir;
		}


	lowsun = sin (nadir / RADIAN);

	midsun = (highsun + lowsun) / 2.0;
	radius = (highsun - lowsun ) / 2.0;

	for (hour = 0; hour < HOURS; hour++)
		{
		rotation = ((hour - (HOURS / 4)) * (360 / HOURS)) / RADIAN;
		sunlevel = (sin (rotation) * radius) + midsun;

		if ((sunlevel > 0.0) && (zenith > 0.0))
			{
			daysun [degree] += (sunlevel *
					   (luminosity *
					   pow ((1.0 / distance), 2.0))) *
					   1.006; /* ref 1.006157; */

				/* constant is correction for
				   average integration error */
			}
		}

	daysun [degree] = daysun [degree] / HOURS;
	}

/*****     *****/

void hemisreport(double sun[], double temp[])
	{
	double repttemp;
	int ctr, lat;

	fprintf (planet, "\n");

	fprintf (planet, "Latitude");
	for (ctr = 0; ctr < 9; ctr++)
		{
		lat = (ctr * 10) + 95;
		fprintf (planet, " %2i", (lat - 90));
		if (inclset > 0.0)
			fprintf (planet, " N   ");
		else
			fprintf (planet, " S   ");
		}
	fprintf (planet, "\n\n");

	fprintf (planet, "Insol:  ");
	for (ctr = 0; ctr < 9; ctr++)
		{
		lat = (ctr * 10) + 95;
		fprintf (planet, " %4.2lf   ", sun [lat]);
		}
	fprintf (planet, "\n");

	fprintf (planet, "Temp:");

	for (ctr = 0; ctr < 9; ctr++)
		{
		lat = (ctr * 10) + 95;

		repttemp = temp [lat] - CELSIUS;
		fprintf (planet, " %6.1lfC", repttemp);
		}

	fprintf (planet, "\n");
	fprintf (planet, "     ");
	for (ctr = 0; ctr < 9; ctr++)
		{
		lat = (ctr * 10) + 95;
		repttemp = ((temp [lat] - CELSIUS) *
			   (9.0 / 5.0)) + 32.0;
		fprintf (planet, " %6.1lfF", repttemp);
		}
	}

/*****     *****/

void globereport (double sun[], double temp[])
	{
	double repttemp;
	int ctr, lat;

	fprintf (planet, "\n");

	fprintf (planet, "Latitude  ");
	for (ctr = 0; ctr < 19; ctr += 2)
		{
		if ((ctr == 4) || (ctr == 9) || (ctr == 14))
			++ctr;
		if (ctr == 10)
			fprintf (planet, "    ");
		lat = (ctr * 10) + 5;

		fprintf (planet, "  %2i", abs (lat - 90));
		if ((lat - 90) > 0.0)
			fprintf (planet, " N  ");
		else
			fprintf (planet, " S  ");
		}
	fprintf (planet, "\n\n");

	fprintf (planet, "Insol:     ");
	for (ctr = 0; ctr < 19; ctr += 2)
		{
		if ((ctr == 4) || (ctr == 9) || (ctr == 14))
			++ctr;
		if (ctr == 10)
			fprintf (planet, "    ");
		lat = (ctr * 10) + 5;
		fprintf (planet, " %4.2lf   ", sun [lat]);
		}
	fprintf (planet, "\n");

	fprintf (planet, "Temp:   ");

	for (ctr = 0; ctr < 19; ctr += 2)
		{
		if ((ctr == 4) || (ctr == 9) || (ctr == 14))
			++ctr;
		if (ctr == 10)
			fprintf (planet, "    ");
		lat = (ctr * 10) + 5;

		repttemp = temp [lat] - CELSIUS;
		fprintf (planet, " %6.1lfC", repttemp);
		}

	fprintf (planet, "\n");
	fprintf (planet, "        ");
	for (ctr = 0; ctr < 19; ctr += 2)
		{
		if ((ctr == 4) || (ctr == 9) || (ctr == 14))
			++ctr;
		if (ctr == 10)
			fprintf (planet, "    ");
		lat = (ctr * 10) + 5;

		repttemp = ((temp [lat] - CELSIUS) *
			   (9.0 / 5.0)) + 32.0;
		fprintf (planet, " %6.1lfF", repttemp);
		}
	}

/*****     *****/

void heattransfer (double transfer)
	/* meridional heat transfer, emperical model */

	{
	double   tempdiff, heatshift,
		beltchange, nextchange,
		beltheatsink, nextheatsink,
		heatsinkratio, current,
		tempchange [ARC];

	int ctr, lat;


	for (ctr = 0; ctr < 18; ctr++)
		{
		lat = (ctr * 10) + 5;

		tempchange [lat] = 0.0;
		}

	for (ctr = 0; ctr < 17; ctr++)
		{
		/* main body computing temp diff */
		lat = (ctr * 10) + 5;

		beltheatsink = nextheatsink = 1.0;
		if (temperature [lat] < ICETEMP)
			{
			beltheatsink = 0.25 + (landsurf * 0.75);
			beltheatsink *= 1.0; /* ref 0.7 */
			}
		if (temperature [lat + 10] < ICETEMP)
			{
			nextheatsink = 0.25 + (landsurf * 0.75);
			nextheatsink *= 1.0;
			}
			/* reduced heat capacity of
			   ice-covered ocean surface */

		heatsinkratio = beltheatsink / nextheatsink;
			/* relative heat capacity
			   of latitude belts */

		tempdiff = temperature [lat] - temperature [lat + 10];

		heatshift = (transfer * (tempdiff *
			    ((INTERVAL * sqrt (yearlength)) / diameter))) *
			    (((temperature [lat] + temperature [lat + 10]) /
			    2.0) / EARTHTEMP);

			/* direct heat transfer based on constant
			   times average temperature */

		if ((temperature [lat] < ICETEMP) ||
		   (temperature [lat + 10] < ICETEMP))
			current = 0.0;
		else
			current = (1.0 - landsurf) * 0.7;
			/* added heat transfer effect
			   of ocean currents; no effect
			   if sea ice present */

		current *= sqrt (pressure);
			/* reduced wind movement of water;
			   higher wind velocity but
			   lower density yields
			   energy-momentum square root rule */

		current *= heatshift;

		heatshift = (((heatshift * 0.5) * sqrt (pressure)) +
			    ((heatshift * 0.5) * latentheat)) +
			    current;

			/* higher wind speed but lower density;
			   latent heat transfer unaffected */


		heatsinkratio *= belt [ctr] / belt [ctr + 1];
			/* factor in relative areas
			   of latitude belts */

		beltchange = -heatshift / sqrt (heatsinkratio);


		nextchange = heatshift * sqrt (heatsinkratio);
			/* temp change from heat transfer
			   governed by relative surface area
			   and heat capacity */

		/*
		fprintf (planet, "\n");
		fprintf (planet, "Temp diff %3.2lf K \n", tempdiff);
		fprintf (planet, "Potential ");
		fprintf (planet, "heat shift %3.2lf K \n", heatshift);
		fprintf (planet, "Heatsink ratio %3.2lf \n\n", heatsinkratio);

		fprintf (planet, "%2i latitude change = ", lat);
		fprintf (planet, "%3.2lf C \n", beltchange);
		fprintf (planet, "%2i latitude change = ", lat + 10);
		fprintf (planet, "%3.2lf C \n", nextchange);
		*/

		if (fabs (beltchange) > (fabs (tempdiff) *
					(1.0 / (heatsinkratio + 1.0))))
			{
			beltchange = -tempdiff *
				     (1.0 / (heatsinkratio + 1.0));
			}
		if (fabs (nextchange) > (fabs (tempdiff) *
					(heatsinkratio /
					(heatsinkratio + 1.0))))
			{
			nextchange = tempdiff *
				     (heatsinkratio / (heatsinkratio + 1.0));
			}

		/*
		fprintf (planet, "     Adjusted -- \n");
		fprintf (planet, "%2i latitude change = ", lat);
		fprintf (planet, "%3.2lf C \n", beltchange);
		fprintf (planet, "%2i latitude change = ", lat + 10);
		fprintf (planet, "%3.2lf C \n", nextchange);
		*/

		tempchange [lat] += beltchange;

		tempchange [lat + 10] += nextchange;
		}


	for (ctr = 0; ctr < 18; ctr++)
		{
		lat = (ctr * 10) + 5;

		/*
		fprintf (planet, "%5i lat:  ", lat - 90);
		fprintf (planet, "temp change %6.2lf K \n", tempchange [lat]);
		  */

		temperature [lat] += tempchange [lat];

		if (temperature [lat] < 1.0)
			temperature [lat] = 1.0;
		}

	}

/*****     *****/

void co2green()
	/* emperical CO2 greenhouse approximation */
	{
	double greenfactor;

	greenfactor = pow (co2, 0.39);
	co2heat = greenfactor * 5.8;
	}

/*****     *****/

double greenhouse (double surftemp)

	/* emperical guesstimate of greenhouse heating */
	{
	double   logratio, reldensity, watervapor,
		greenfactor, greentemp, othergases;


	othergases = 1.25; /* ref 1.35 */

	if (surftemp < 1.0)
		surftemp = 1.0;

	logratio = log (surftemp / 288.0) /
		   (0.123 * pow ((surftemp / 288.0), 0.6));

		/* empirical "law" based on natural log
		   of temp ratio to produce change in
		   absolute humidity (water vapor partial
		   pressure) as temperature varies */


	reldensity = pow (10.0, logratio);


	watervapor = (reldensity * 16.0) * 0.68;

		/* saturation humidity 16 millibars
		   at standard temperature; 0.68 is
		   inferred average humidity
		   over oceans */

	watervapor *= (1.0 + landsurf) * (1.0 - landsurf);

		/* emperical formula based on relative
		   ocean and continental precipitation;
		   ocean humidity at standard value,
		   continental humidity declines in
		   proportion to land surface */

	/*
	printw ("Water vapor, %6.2lf mb \n", watervapor);
	refresh();
	*/


	if (watervapor > (STDPRESSURE *
			 (500.0 * pow ((1.0 - landsurf), 1.5))))
		/* oceans fully evaporated,
		   no more water vapor available;
		   assumption that ocean depth
		   is proportional to square root
		   of surface area */
		{
		watervapor = STDPRESSURE *
			     (500.0 * pow ((1.0 - landsurf), 1.5));
		}


	greenfactor = pow (watervapor, 0.39);

	greentemp = (greenfactor * 11.0) + (co2heat + othergases);

	return (greentemp);
	}

/*****     *****/

double weather (double surftemp)

	/* emperical guesstimate of greenhouse heating */
	{
	double   logratio, relvapor, cloudcover, reflect;


	if (surftemp < 1.0)
		surftemp = 1.0;

	logratio = log (surftemp / 288.0) /
		   (0.123 * pow ((surftemp / 288.0), 0.6));

	relvapor = pow (10.0, logratio);

		/* relvapor x 16 = saturation water vapor
		   in millibars at standard temperature
		   (note: inferred oceanic humidity is 0.68) */


	relvapor *= ((1.0 + landsurf) * (1.0 - landsurf)) / 0.92;

		/* emperical formula based on relative
		   ocean and continental precipitation;
		   0.92 is Earth's relative humidity,
		   compared to a thalassic planet */


	latentheat = pow (relvapor, latentfactor);

	cloudcover = latentheat * 0.47;
	if (cloudcover > 1.0)
		cloudcover = 1.0;

	reflect = (latentheat * 0.26) + 0.20;
	if (reflect > 0.8)
		reflect = 0.8;

	cloudalbedo = cloudcover * reflect;

	albedo = cloudalbedo +
		 ((1.0 - cloudcover) * surfalbedo);

	if (albedo < surfalbedo)
		albedo = surfalbedo;

	return (cloudcover * 100.0);
	}

/*****     *****/

void ellipse()
	/* model an ellipse */

	{
	double   baserad, keplerrad, monthangle, theta, sunangle;

	int monthset;

	if (initflag)
		{
		fprintf (planet, "      Time                  ");
		fprintf (planet, "Distance           Position \n\n");
		}

	for (monthset = 1; monthset <= (YEAR / MONTH); monthset++)
		{
		monthangle = monthset * 30.0;

		theta = monthangle + precession;

		baserad = (1.0 - pow (eccentricity, 2.0)) /
			  (1.0 + (eccentricity * cos (theta / RADIAN)));

		keplerrad = 2.0 - baserad;

		sunangle = asin ((baserad * sin (theta / RADIAN)) /
			   keplerrad);

		if ((cos (theta / RADIAN) * baserad) < -(2.0 * eccentricity))
			sunangle = PI - sunangle;

		if (sunangle < 0.0)
			sunangle = (2.0 * PI) + sunangle;

		sunangle *= RADIAN;


		if (initflag)
			{
			fprintf (planet, " %9.2lf years: ",
			((monthset * yearlength) / 12.0));

			fprintf (planet, "       ");
			fprintf (planet, "%8.3lf AU ", (keplerrad *
						      orbitdist));

			fprintf (planet, "          ");
			fprintf (planet, "%3.0lf deg \n", sunangle);
			}

		distance [monthset] = keplerrad;

		incline [monthset] = sin (((sunangle - precession) +
				     270.0) / RADIAN) * axis;
		}

	}

/** END **/

